"""
Backend for grades.
"""

from lms.djangoapps.grades.course_grade_factory import CourseGradeFactory  # pylint: disable=import-error


def get_course_grade_factory():
    """ get CourseGradeFactory object. """
    return CourseGradeFactory



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Test module for CourseKey validations under the open-release/hawthorn.beta1 tag
"""
from django.test import TestCase
from mock import patch
from rest_framework.serializers import ValidationError

from ..serializers import EdxappCourseEnrollmentSerializer


class CourseKeyValidationTest(TestCase):
    """ Tests for the CourseKey validations """

    def setUp(self):
        """ setup """
        super().setUp()
        self.enrollment_serializer = EdxappCourseEnrollmentSerializer
        self.m_enrollment = {
            'mode': 'audit',
            'user': 'test',
            'is_active': True,
        }

    @patch('eox_core.api.v1.serializers.validate_org')
    @patch('eox_core.api.v1.serializers.check_edxapp_enrollment_is_valid', return_value=[])
    @patch('eox_core.api.v1.serializers.get_valid_course_key')
    def test_incorrect_course_id(self, m_get_valid_course_key, *_):
        """ Test that the CourseKey validation fails due to invalid formatting """
        course_id = "test1234"
        m_get_valid_course_key.side_effect = ValidationError(f"Invalid course_id {course_id}")
        self.m_enrollment['course_id'] = course_id
        serializer = self.enrollment_serializer(data=self.m_enrollment)
        with self.assertRaises(ValidationError):
            serializer.is_valid(raise_exception=True)
        m_get_valid_course_key.assert_called_once_with(course_id)

    @patch('eox_core.api.v1.serializers.validate_org')
    @patch('eox_core.api.v1.serializers.get_valid_course_key')
    @patch('eox_core.api.v1.serializers.check_edxapp_enrollment_is_valid', return_value=[])
    def test_course_id_valid_org(self, _, m_get_valid_course_key, m_validate_org):
        """ Test that the CourseKey validation fails because it doesn't belong to the filtered org """
        course_id = "course-v1:org+course+run"
        m_get_valid_course_key.return_value = None
        m_validate_org.return_value = False

        self.m_enrollment['course_id'] = course_id
        serializer = self.enrollment_serializer(data=self.m_enrollment)
        with self.assertRaises(ValidationError):
            serializer.is_valid(raise_exception=True)

        m_validate_org.assert_called_once_with(course_id)

    @patch('eox_core.api.v1.serializers.validate_org')
    @patch('eox_core.api.v1.serializers.get_valid_course_key')
    @patch('eox_core.api.v1.serializers.check_edxapp_enrollment_is_valid', return_value=[])
    def test_course_id_correct(self, _, m_get_valid_course_key, m_validate_org):
        """ Test that the CourseKey validation works under normal conditions """
        course_id = "course-v1:org+course+run"
        m_get_valid_course_key.return_value = None
        m_validate_org.return_value = True

        self.m_enrollment['course_id'] = course_id
        serializer = self.enrollment_serializer(data=self.m_enrollment)

        serializer.is_valid(raise_exception=True)

        m_validate_org.assert_called_once_with(course_id)
        m_get_valid_course_key.assert_called_once_with(course_id)



""" urls.py """

from django.conf import settings
from django.urls import re_path

from eox_core.api.v1 import views

app_name = 'eox_core'  # pylint: disable=invalid-name

urlpatterns = [  # pylint: disable=invalid-name
    re_path(r'^user/$', views.EdxappUser.as_view(), name='edxapp-user'),
    re_path(r'^enrollment/$', views.EdxappEnrollment.as_view(), name='edxapp-enrollment'),
    re_path(r'^grade/$', views.EdxappGrade.as_view(), name='edxapp-grade'),
    re_path(r'^pre-enrollment/$', views.EdxappPreEnrollment.as_view(), name='edxapp-pre-enrollment'),
    re_path(r'^userinfo/$', views.UserInfo.as_view(), name='edxapp-userinfo'),
]

if getattr(settings, "EOX_CORE_ENABLE_UPDATE_USERS", None):
    urlpatterns += [
        re_path(r'^update-user/$', views.EdxappUserUpdater.as_view(), name='edxapp-user-updater'),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Storages backend
"""
from openedx.core.storage import DevelopmentStorage, ProductionStorage  # pylint: disable=import-error


def get_edxapp_production_staticfiles_storage():  # pylint: disable=invalid-name
    """
    Return the edx-platform production staticfiles storage
    """
    return ProductionStorage


def get_edxapp_development_staticfiles_storage():  # pylint: disable=invalid-name
    """
    Return the edx-platform development staticfiles storage
    """
    return DevelopmentStorage



"""
Integration test suite.

This suite performs multiple http requests to guarantee
that the Grade API is behaving as expected on a live server.
"""
import json
from os import environ

import pytest
import requests
from django.test import TestCase
from rest_framework import status


@pytest.mark.skipif(not environ.get("TEST_INTEGRATION"), reason="Run only explicitly")
class TesteGradeIntegration(TestCase):  # pragma: no cover
    """Test suite"""

    data = {}

    @classmethod
    def setUpClass(cls):
        with open("eox_core/tests/integration/test_data", encoding="utf-8") as file_obj:
            cls.data = json.load(file_obj)
        cls.data["endpoint"] = "eox-core/api/v1/grade/"
        site1_data = {
            "client_id": cls.data["site1_data"]["client_id"],
            "client_secret": cls.data["site1_data"]["client_secret"],
            "grant_type": "client_credentials",
        }
        site2_data = {
            "client_id": cls.data["site2_data"]["client_id"],
            "client_secret": cls.data["site2_data"]["client_secret"],
            "grant_type": "client_credentials",
        }
        request_url = f"{cls.data['site1_data']['base_url']}/oauth2/access_token/"
        response_site1 = requests.post(request_url, data=site1_data, timeout=10)
        response_site1.raise_for_status()
        cls.data["site1_data"]["token"] = response_site1.json()["access_token"]
        request_url = f"{cls.data['site2_data']['base_url']}/oauth2/access_token/"
        response_site2 = requests.post(request_url, data=site2_data, timeout=10)
        response_site2.raise_for_status()
        cls.data["site1_data"]["token"] = response_site1.json()["access_token"]
        cls.data["site2_data"]["token"] = response_site2.json()["access_token"]

        create_enrollment(cls.data)

    @classmethod
    def tearDownClass(cls):
        pass

    def test_read_valid_default_params(self):
        # pylint: disable=invalid-name
        """
        Get grades info from a user enrolled on a course without the optional
        fields.
        """
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.get(request_url, data=data, headers=headers, timeout=10)
        response_content = response.json()

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn("earned_grade", response_content)
        self.assertNotIn("grading_policy", response_content)
        self.assertNotIn("section_breakdown", response_content)

    def test_read_detail_no_policy(self):
        # pylint: disable=invalid-name
        """
        Get grades info from a user enrolled on a course. Include detailed info
        for each graded subsection.
        """
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "detailed": True,
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.get(request_url, data=data, headers=headers, timeout=10)
        response_content = response.json()

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn("earned_grade", response_content)
        self.assertIn("section_breakdown", response_content)
        self.assertNotIn("grading_policy", response_content)

    def test_read_policy_detail(self):
        # pylint: disable=invalid-name
        """
        Get grades info from a user enrolled on a course. Include all extra info
        (subsection details and grading policy)
        """
        site1_data = self.data["site1_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "detailed": True,
            "grading_policy": True,
        }
        headers = {
            "Authorization": f"Bearer {site1_data['token']}",
            "Host": site1_data["host"],
        }
        request_url = f"{site1_data['base_url']}/{self.data['endpoint']}"

        response = requests.get(request_url, data=data, headers=headers, timeout=10)
        response_content = response.json()

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertIn("earned_grade", response_content)
        self.assertIn("section_breakdown", response_content)
        self.assertIn("grading_policy", response_content)
        self.assertIn("grader", response_content["grading_policy"])
        self.assertIn("grade_cutoffs", response_content["grading_policy"])

    def test_read_invalid_enrollment_for_site(self):
        # pylint: disable=invalid-name
        """
        Get grade info for a user and course from another site.
        """
        site1_data = self.data["site1_data"]
        site2_data = self.data["site2_data"]
        data = {
            "email": site1_data["user_email"],
            "course_id": site1_data["course"]["id"],
            "detailed": True,
            "grading_policy": True,
        }
        headers = {
            "Authorization": f"Bearer {site2_data['token']}",
            "Host": site2_data["host"],
        }
        request_url = f"{site2_data['base_url']}/{self.data['endpoint']}"

        response = requests.get(request_url, data=data, headers=headers, timeout=10)
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)


def create_enrollment(data):
    """
    Auxiliary function to setUp test fixtures. Creates/enables a new enrollment.

    :param data: dictionary with all the parameters needed to create an enrollment.
    """
    req_data = {
        "email": data["site1_data"]["user_email"],
        "course_id": data["site1_data"]["course"]["id"],
        "mode": data["site1_data"]["course"]["mode"],
    }
    headers = {
        "Authorization": f"Bearer {data['site1_data']['token']}",
        "Host": data["site1_data"]["host"],
    }
    request_url = f"{data['site1_data']['base_url']}/eox-core/api/v1/enrollment/"
    response = requests.post(request_url, data=req_data, headers=headers, timeout=10)
    response.raise_for_status()



"""
Test module for the Task dispatcher API
"""
from django.contrib.auth.models import User
from django.test import TestCase, override_settings
from django.urls import reverse
from mock import MagicMock, patch
from rest_framework import status
from rest_framework.test import APIClient

from eox_core.api.task_dispatcher.v1.views import TaskAPI


@override_settings(EOX_CORE_ASYNC_TASKS=["dummy.task.path.task_name"])
class TestTaskAPI(TestCase):
    """ Tests for the task dispatcher API """

    patch_permissions = patch(
        "rest_framework.permissions.IsAdminUser.has_permission",
        return_value=True,
    )

    def setUp(self):
        """setup fixtures"""
        self.api_user = User(1, "test@example.com", "test")
        self.client = APIClient()
        self.client.force_authenticate(user=self.api_user)
        self.url = reverse("eox-task-api:task-api")

    @patch("celery.current_app.AsyncResult")
    @patch_permissions
    def test_get_task_statuses(self, _, m_async_result):
        """Test that the GET method works with default parameters"""
        m_async_result.return_value.status = "READY"
        m_async_result.return_value.result = "result"
        m_async_result.return_value.ready.return_value = True
        expected_response = {"result": "result", "status": "READY"}

        response = self.client.get(self.url, data={"id": "2"})

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, expected_response)

    @patch.object(TaskAPI, "import_task")
    @patch("celery.current_app")
    @patch('eox_core.api.task_dispatcher.v1.views.get_current_site')
    @patch_permissions
    def test_post_valid_task(self, _, m_site, m_current_app, m_task_function):
        """
        Test that the POST method works for a task that is enabled in the
        EOX_CORE_ASYNC_TASKS setting and registered in the worker
        """
        result = MagicMock()
        result.id = "70-421"
        m_task_function.return_value.delay.return_value = result
        m_current_app.tasks = ["dummy.task.path.task_name"]
        m_site.id = 1
        params = {
            "task_path": "dummy.task.path.task_name",
            "task_params": {},
        }
        expected_response = {"task_id": result.id}

        response = self.client.post(self.url, data=params, format="json")

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data, expected_response)

    @patch("celery.current_app")
    @patch('eox_core.api.task_dispatcher.v1.views.get_current_site')
    @patch_permissions
    def test_post_disabled_task(self, _, m_site, m_current_app):
        """
        Test that the POST method fails for a task that is registered in the
        in the worker and is disabled in EOX_CORE_ASYNC_TASKS.
        """
        m_current_app.tasks = ["dummy.task.path.task_name2"]
        m_site.id = 1
        params = {
            "task_path": "dummy.task.path.task_name2",
            "task_params": {},
        }
        expected_response = {"detail": "Invalid task path"}

        response = self.client.post(self.url, data=params, format="json")

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(response.data, expected_response)

    @patch("celery.current_app")
    @patch('eox_core.api.task_dispatcher.v1.views.get_current_site')
    @patch_permissions
    def test_post_unregistered_task(self, _, m_site, m_current_app):
        """
        Test that the POST method fails for a task that is enabled in
        EOX_CORE_ASYNC_TASKS and is **not** registered in the worker
        """
        m_current_app.tasks = ["dummy.task.path.task_name2"]
        m_site.id = 1
        params = {
            "task_path": "dummy.task.path.task_name",
            "task_params": {},
        }
        expected_response = {"detail": "Invalid task path"}

        response = self.client.post(self.url, data=params, format="json")

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(response.data, expected_response)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Test module for the permissions class
"""
from django.test import TestCase
from mock import MagicMock
from rest_framework.exceptions import NotAuthenticated

from ..permissions import EoxCoreAPIPermission


class PermissionsTest(TestCase):
    """ Tests for the API permissions """

    def test_permissions_for_staff(self):
        """ Staff always passes """
        request = MagicMock()
        request.user.is_staff = True

        has_perm = EoxCoreAPIPermission().has_permission(request, MagicMock())

        self.assertTrue(has_perm)

    def test_read_user_permissions(self):
        """ If the auth does not fail, it comes down to the has_permission result """
        request = MagicMock()
        request.user.is_staff = False
        request.user.has_perm.return_value = False

        has_perm = EoxCoreAPIPermission().has_permission(request, MagicMock())

        self.assertFalse(has_perm)

    def test_permissions_without_auth(self):
        """ If anything in the auth fails, the NotAuthenticated exception is raised """
        request = MagicMock()
        request.user.is_staff = False
        request.user.has_perm.return_value = False
        request.auth = None

        with self.assertRaises(NotAuthenticated):
            EoxCoreAPIPermission().has_permission(request, MagicMock())

    def test_permissions_auth_dop(self):
        """ Authorize the domain via the client.url """
        request = MagicMock()
        request.user.is_staff = False
        request.user.has_perm.return_value = True
        request.get_host.return_value = "domain.com"
        request.auth.client.url = "https://domain.com/"

        has_perm = EoxCoreAPIPermission().has_permission(request, MagicMock())

        self.assertTrue(has_perm)

    def test_permissions_auth_dot(self):
        """ Authorize the domain via the application.allowed_uris """
        request = MagicMock()
        request.user.is_staff = False
        request.user.has_perm.return_value = True
        request.auth.application.redirect_uri_allowed.return_value = True

        has_perm = EoxCoreAPIPermission().has_permission(request, MagicMock())

        self.assertTrue(has_perm)



""" Utils for testing"""
from datetime import datetime

import factory
from django.contrib.auth.models import User

DEFAULT_PASSWORD = 'test'


class SuperUserFactory(factory.django.DjangoModelFactory):
    """
    A Factory for User objects.
    """
    class Meta:
        """ Meta """
        model = User
        django_get_or_create = ('email', 'username')

    _DEFAULT_PASSWORD = 'test'

    username = factory.Sequence(lambda n: f'robot{n}')
    email = factory.Sequence(lambda n: f'robot+test+{n}@example.com')
    password = factory.PostGenerationMethodCall(
        'set_password', _DEFAULT_PASSWORD)
    first_name = factory.Sequence(lambda n: f'Robot{n}')
    last_name = 'Test'
    is_staff = True
    is_active = True
    is_superuser = True
    last_login = datetime(2012, 1, 1)
    date_joined = datetime(2011, 1, 1)


class TestStorage:
    """
    This is a storage used for testing purposes
    """
    def url(self, name):
        """
        return the name of the asset
        """
        return name



""" Module for the cs_comments_service User object."""
from openedx.core.djangoapps.django_comment_common.comment_client.user import User  # pylint: disable=import-error


def replace_username_cs_user(*args, **kwargs):
    """
    Replace user's username in cs_comments_service (forums).

    kwargs:
        user: edxapp user whose username is being replaced.
        new_username: new username.
    """
    user = kwargs.get("user")
    new_username = kwargs.get("new_username")

    cs_user = User.from_django_user(user)
    cs_user.replace_username(new_username)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
CourseKey public function definitions
"""

from importlib import import_module

from django.conf import settings


def get_valid_course_key(course_id):
    """
    Return a valid CourseKey for the given course_id
    """

    backend_function = settings.EOX_CORE_COURSEKEY_BACKEND
    backend = import_module(backend_function)

    return backend.get_valid_course_key(course_id)


def validate_org(course_id):
    """
    Return a valid CourseKey for the given course_id
    """

    backend_function = settings.EOX_CORE_COURSEKEY_BACKEND
    backend = import_module(backend_function)

    return backend.validate_org(course_id)



""" Backend abstraction. """
from importlib import import_module

from django.conf import settings


def get_configuration_helper(*args, **kwargs):
    """ Get configuration helper module """
    backend_function = settings.EOX_CORE_CONFIGURATION_HELPER_BACKEND
    backend = import_module(backend_function)
    return backend.get_configuration_helper(*args, **kwargs)



"""
Settings for eox_core project meant to be called on the edx-platform/*/envs/devstack.py module
"""
from .production import *  # pylint: disable=wildcard-import, unused-wildcard-import


def plugin_settings(settings):  # pylint: disable=function-redefined
    """
    Set of plugin settings used by the Open Edx platform.
    More info: https://github.com/edx/edx-platform/blob/master/openedx/core/djangoapps/plugins/README.rst
    """
    settings.EOX_CORE_STATICFILES_STORAGE = "eox_core.storage.DevelopmentStorage"
    if settings.EOX_CORE_ENABLE_STATICFILES_STORAGE:
        settings.STATICFILES_STORAGE = settings.EOX_CORE_STATICFILES_STORAGE



"""
The pipeline module defines functions that are used in the third party authentication flow
"""
import logging

from crum import get_current_request
from django.db.models.signals import post_save
from social_core.exceptions import AuthFailed, NotAllowedToDisconnect

from eox_core.edxapp_wrapper.users import (
    generate_password,
    get_user_attribute,
    get_user_profile,
    get_user_signup_source,
)
from eox_core.logging import logging_pipeline_step

UserSignupSource = get_user_signup_source()  # pylint: disable=invalid-name
LOG = logging.getLogger(__name__)


# pylint: disable=unused-argument,keyword-arg-before-vararg
def ensure_new_user_has_usable_password(backend, user=None, **kwargs):
    """
    This pipeline function assigns an usable password to an user in case that the user has an unusable password on user cretion.
    At the creation of new users through some TPA providers, some of them are created with an unusable password, a user with an unusable password cannot login
    properly in the platform if the common.djangoapps.third_party.pipeline.set_logged_in_cookies step is enabled.

    See: https://github.com/eduNEXT/edunext-platform/blob/c83b82a4aba2a5496e3b7da83972a8edf25fcdd2/common/djangoapps/third_party_auth/pipeline.py#L673

    It's recommended to place this step after the social core step that creates the users: (social_core.pipeline.user.create_user).
    """
    is_new = kwargs.get('is_new')

    if user and is_new and not user.has_usable_password():
        user.set_password(generate_password(length=25))
        user.save()

        user_attribute_model = get_user_attribute()
        user_attribute_model.set_user_attribute(user, 'auto_password_via_tpa_pipeline', 'true')

        logging_pipeline_step(
            "info",
            "Assigned an usable password to the user on creation.",
            **locals()
        )


# pylint: disable=unused-argument,keyword-arg-before-vararg
def ensure_user_has_profile(backend, details, user=None, *args, **kwargs):
    """
    This pipeline function creates an empty profile object if the user does not have one.
    It can be used with the user_details_force_sync function to fill the profile after creation.
    """
    if user:
        user_profile_model = get_user_profile()
        try:
            __ = user.profile
        except user_profile_model.DoesNotExist:
            user_profile_model.objects.create(user=user)
            logging_pipeline_step(
                "info",
                "Created new profile for user during the third party pipeline.",
                **locals()
            )


def force_user_post_save_callback(user=None, *args, **kwargs):
    """
    Send the signal post_save in order to force the execution of user_post_save_callback, see it in
    https://github.com/eduNEXT/edunext-platform/blob/4169327231de00991c46b6192327fe50b0406561/common/djangoapps/student/models.py#L652
    this allows the automatic enrollments if a user is registered by a third party auth.

    It's recommended to place this step after the social core step that creates the users: (social_core.pipeline.user.create_user).

    the discussion left three options:

    1)
        user._changed_fields = {'is_active': user.is_active}  # pylint: disable=protected-access
        post_save.send_robust(
            sender=user.__class__,
            instance=user,
            created=False
        )

    2)
        user.is_active = False
        user.save()
        user.is_active = True
        user.save()

    3)
        registration = Registration.objects.get_or_create(user=user)
        user.is_active = False
        user.save()
        registration.activate()

    The first one was selected because the second executed multiple unnecessary process and third one
    needs to implement a new backend.
    """
    is_new = kwargs.get('is_new')

    if user and is_new:
        user._changed_fields = {'is_active': user.is_active}  # pylint: disable=protected-access
        post_save.send_robust(
            sender=user.__class__,
            instance=user,
            created=False
        )
        logging_pipeline_step(
            "info",
            "Post_save signal sent in order to force the execution of user_post_save_callback.",
            **locals()
        )


def check_disconnect_pipeline_enabled(backend, *args, **kwargs):
    """
    This pipeline function checks whether disconnection from the auth provider is enabled or not. That's
    done checking for `disableDisconnectPipeline` setting defined in the provider configuration.

    For example:
        To avoid disconnection from SAML, add the following to `Other config str` in your SAMLConfiguration:

        "BACKEND_OPTIONS": { "disableDisconnectPipeline":true },

        Now, to avoid disconnection from an Oauth2Provider, add the same setting to `Other settings` in your
        Oauth2Provider.

    It's recommended to place this function at the beginning of the pipeline.
    """
    if backend and backend.setting("BACKEND_OPTIONS", {}).get("disableDisconnectPipeline"):
        logging_pipeline_step(
            "error",
            "Disconnection pipeline is disabled, users are not allowed to disconnect.",
            **locals()
        )
        raise NotAllowedToDisconnect()  # pylint: disable=raising-non-exception


def assert_user_information(details, user, backend, *args, **kwargs):
    """
    This pipeline function checks whether the user information from the LMS matches the information
    returned by the provider.

    For example:
        To avoid connection when the user's email from the LMS differs from the email returned by the provider,
        add the setting:

        "BACKEND_OPTIONS": { "assertUserInformationMatchFields":["email",...] }

    It's recommended to place this step after the user is available (ie. after ensure_user_information step) and
    before user association (ie. before user_association step)
    """
    if not (user and backend):
        return

    defined_fields = backend.setting("BACKEND_OPTIONS", {}).get("assertUserInformationMatchFields", [])

    for field in defined_fields:

        if str(details.get(field, "")) != str(getattr(user, field, "")):
            logging_pipeline_step(
                "error",
                f"Credentials not allowed: field {field} returned by provider does not match with the users information.",
                **locals()
            )
            raise AuthFailed(backend, "Credentials not allowed.")  # pylint: disable=raising-non-exception


def create_signup_source_for_new_association(user=None, *args, **kwargs):
    """
    This pipeline step registers a new signup source for users with new social auth link. The signup source
    will have associated the current site and user.

    It's recommended to place this step after the creation of the social auth link, i.e, after the step
    'social_core.pipeline.social_auth.associate_user',

    To avoid errors or unnecessary searches, these steps are skipped using 'is_new' key insterted by
    'social_core.pipeline.user.create_user' after creating a user.
    """
    if not user or kwargs.get("is_new", False):
        return

    if kwargs.get("new_association", False):
        _, created = UserSignupSource.objects.get_or_create(user=user, site=get_current_request().site)
        if created:
            logging_pipeline_step(
                "info",
                "Created new singup source for user during the third party pipeline.",
                **locals()
            )


def ensure_user_has_signup_source(user=None, *args, **kwargs):
    """
    This pipeline step registers a new signup source for users that have a social auth link but
    no signup source associated with the current site. The signup source will be associated to
    the current site and user.

    It's recommended to place this step at the end of the TPA pipeline, e.g., after the step
    'eox_core.pipeline.ensure_user_has_profile',

    To avoid errors or unnecessary searches, these steps are skipped using 'is_new' key insterted by
    'social_core.pipeline.user.create_user' after creating a user.
    """
    if not user or kwargs.get("is_new", False):
        return

    _, created = UserSignupSource.objects.get_or_create(user=user, site=get_current_request().site)
    if created:
        logging_pipeline_step(
            "info",
            "Created new singup source for user during the third party pipeline.",
            **locals()
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Backend for the create_edxapp_user that works under the open-release/lilac.master tag
"""
# pylint: disable=import-error, protected-access
import datetime
import logging

from common.djangoapps.course_modes.models import CourseMode
from common.djangoapps.student.models import CourseEnrollment
from django.contrib.auth.models import User
from opaque_keys import InvalidKeyError
from opaque_keys.edx.keys import CourseKey
from openedx.core.djangoapps.content.course_overviews.models import CourseOverview
from openedx.core.djangoapps.enrollments import api  # pylint: disable=ungrouped-imports
from openedx.core.djangoapps.enrollments.errors import (  # pylint: disable=ungrouped-imports
    CourseEnrollmentExistsError,
    CourseModeNotFoundError,
)
from openedx.core.lib.exceptions import CourseNotFoundError
from pytz import utc
from rest_framework.exceptions import APIException, NotFound

from eox_core.edxapp_wrapper.backends.edxfuture_i_v1 import get_program
from eox_core.edxapp_wrapper.coursekey import get_valid_course_key, validate_org
from eox_core.edxapp_wrapper.users import check_edxapp_account_conflicts

LOG = logging.getLogger(__name__)


def create_enrollment(user, *args, **kwargs):
    """
    backend function to create enrollment

    Example:
        >>>create_enrollment(
            user_object,
            {
            "course_id": "course-v1-edX-DemoX-1T2015",
            ...
            }
        )

    """
    kwargs = dict(kwargs)
    program_uuid = kwargs.pop('bundle_id', None)
    course_id = kwargs.pop('course_id', None)

    if program_uuid:
        return _enroll_on_program(user, program_uuid, *args, **kwargs)
    if course_id:
        return _enroll_on_course(user, course_id, *args, **kwargs)

    raise APIException("You have to provide a course_id or bundle_id")


def update_enrollment(user, course_id, mode, *args, **kwargs):
    """
    Update enrollment of given user in the course provided.

    Example:
        >>>update_enrollment(
            user_object,
            course_id,
            mode,
            is_active=False,
            enrollment_attributes=[
                {
                    "namespace": "credit",
                    "name": "provider_id",
                    "value": "hogwarts",
                },
                {...}
                ]
            }
        )
    """
    username = user.username

    is_active = kwargs.get('is_active', True)
    enrollment_attributes = kwargs.get('enrollment_attributes', None)

    LOG.info('Updating enrollment for student: %s of course: %s mode: %s', username, course_id, mode)
    enrollment = api._data_api().update_course_enrollment(username, course_id, mode, is_active)
    if not enrollment:
        raise NotFound(f'No enrollment found for {username}')
    if enrollment_attributes is not None:
        api.set_enrollment_attributes(username, course_id, enrollment_attributes)

    return {
        'user': enrollment['user'],
        'course_id': course_id,
        'mode': enrollment['mode'],
        'is_active': enrollment['is_active'],
        'enrollment_attributes': enrollment_attributes,
    }


def get_enrollment(*args, **kwargs):
    """
    Return enrollment of given user in the course provided.

    Example:
        >>>get_enrollment(
            {
            "username": "Bob",
            "course_id": "course-v1-edX-DemoX-1T2015"
            }
        )
    """
    errors = []
    course_id = kwargs.pop('course_id', None)
    username = kwargs.get('username', None)

    try:
        LOG.info('Getting enrollment information of student: %s  course: %s', username, course_id)
        enrollment = api.get_enrollment(username, course_id)
        if not enrollment:
            errors.append(f'No enrollment found for user:`{username}`')
            return None, errors
    except InvalidKeyError:
        errors.append(f'No course found for course_id `{course_id}`')
        return None, errors
    enrollment['enrollment_attributes'] = api.get_enrollment_attributes(username, course_id)
    enrollment['course_id'] = course_id
    return enrollment, errors


def delete_enrollment(*args, **kwargs):
    """
    Delete enrollment and enrollment attributes of given user in the course provided.

    Example:
        >>>delete_enrollment(
            {
            "user": user_object,
            "course_id": course-v1-edX-DemoX-1T2015"
        )
    """
    course_id = kwargs.pop('course_id', None)
    user = kwargs.get('user')
    try:
        course_key = get_valid_course_key(course_id)
    except InvalidKeyError:
        raise NotFound(f'No course found by course id `{course_id}`') from InvalidKeyError

    username = user.username

    LOG.info('Deleting enrollment. User: `%s`  course: `%s`', username, course_id)
    enrollment = CourseEnrollment.get_enrollment(user, course_key)
    if not enrollment:
        raise NotFound(f'No enrollment found for user: `{username}` on course_id `{course_id}`')
    try:
        enrollment.delete()
    except Exception:
        raise NotFound(f'Error: Enrollment could not be deleted for user: `{username}` on course_id `{course_id}`') from Exception


def _enroll_on_course(user, course_id, *args, **kwargs):
    """
    enroll user on a single course

    Example:
        >>>_enroll_on_course(
            {
            "user": user_object,
            "course_id": course-v1-edX-DemoX-1T2015",
            "is_active": "False",
            "mode": "audit",
            "enrollment_attributes": [
                {
                    "namespace": "credit",
                    "name": "provider_id",
                    "value": "hogwarts",
                },
                {...}
                ]
            }
        )
    """
    errors = []

    username = user.username

    mode = kwargs.get('mode', 'audit')
    is_active = kwargs.get('is_active', True)
    force = kwargs.get('force', False)
    enrollment_attributes = kwargs.get('enrollment_attributes', None)

    enrollment_valid_query = {
        'course_id': course_id,
        'force': force,
        'mode': mode,
        'username': username,
    }
    validation_errors = check_edxapp_enrollment_is_valid(**enrollment_valid_query)
    if validation_errors:
        return None, [", ".join(validation_errors)]

    try:
        LOG.info('Creating regular enrollment %s, %s, %s', username, course_id, mode)
        enrollment = _create_or_update_enrollment(username, course_id, mode, is_active, force)
    except CourseNotFoundError as err:
        raise NotFound(repr(err)) from err
    except Exception as err:  # pylint: disable=broad-except
        if force:
            LOG.info('Force create enrollment %s, %s, %s', username, course_id, mode)
            enrollment = _force_create_enrollment(username, course_id, mode, is_active)
        else:
            err_msg = None
            if not str(err):
                err_msg = err.__class__.__name__
            raise APIException(detail=err_msg if err_msg else err) from err

    if enrollment_attributes is not None:
        api.set_enrollment_attributes(username, course_id, enrollment_attributes)
    try:
        enrollment['enrollment_attributes'] = enrollment_attributes
        enrollment['course_id'] = course_id
    except TypeError:
        pass
    return enrollment, errors


def _enroll_on_program(user, program_uuid, *arg, **kwargs):
    """
    enroll user on each of the courses of a program
    """
    results = []
    errors = []
    LOG.info('Enrolling on program: %s', program_uuid)
    try:
        data = get_program(program_uuid)
    except Exception as err:  # pylint: disable=broad-except
        raise NotFound(repr(err)) from err
    if not data['courses']:
        raise NotFound("No courses found for this program")
    for course in data['courses']:
        if course['course_runs']:
            course_run = _get_preferred_course_run(course)
            LOG.info('Enrolling on course_run: %s', course_run['key'])
            course_id = course_run['key']
            try:
                result, errors_list = _enroll_on_course(user, course_id, *arg, **kwargs)
            except APIException as error:
                result = {
                    'username': user.username,
                    'mode': None,
                    'course_id': course_id,
                }
                errors_list = [error.detail]

            results.append(result)
            errors.append(errors_list)
        else:
            raise NotFound("No course runs available for this course")
    return results, errors


def _get_preferred_course_run(course):
    """
    Returns the course run more likely to be the intended one
    """
    sorted_course_runs = sorted(course['course_runs'], key=lambda run: run['start'])

    for run in sorted_course_runs:
        default_enrollment_start_date = datetime.datetime(1900, 1, 1, tzinfo=utc)
        course_run_key = CourseKey.from_string(run['key'])
        course_overview = CourseOverview.get_from_id(course_run_key)
        enrollment_end = course_overview.enrollment_end or datetime.datetime.max.replace(tzinfo=utc)
        enrollment_start = course_overview.enrollment_start or default_enrollment_start_date
        run['is_enrollment_open'] = enrollment_start <= datetime.datetime.now(utc) < enrollment_end

    open_course_runs = [run for run in sorted_course_runs if run['is_enrollment_open']]
    course_run = open_course_runs[0] if open_course_runs else sorted_course_runs[-1]
    return course_run


# pylint: disable=invalid-name
def check_edxapp_enrollment_is_valid(*args, **kwargs):
    """
    backend function to check if enrollment is valid
    """
    errors = []
    is_active = kwargs.get("is_active", True)
    course_id = kwargs.get("course_id")
    force = kwargs.get('force', False)
    mode = kwargs.get("mode")
    program_uuid = kwargs.get('bundle_id')
    username = kwargs.get("username")
    email = kwargs.get("email")

    if program_uuid and course_id:
        return ['You have to provide a course_id or bundle_id but not both']
    if not program_uuid and not course_id:
        return ['You have to provide a course_id or bundle_id']
    if not email and not username:
        return ['Email or username needed']
    if not check_edxapp_account_conflicts(email=email, username=username):
        return ['User not found']
    if mode not in CourseMode.ALL_MODES:
        return ['Invalid mode given:' + mode]
    if course_id:
        if not validate_org(course_id):
            errors.append('Enrollment not allowed for given org')
    if course_id and not force:
        try:
            api.validate_course_mode(course_id, mode, is_active=is_active)
        except CourseModeNotFoundError:
            errors.append('Mode not found')
        except CourseNotFoundError:
            errors.append('Course not found')
    return errors


def _create_or_update_enrollment(username, course_id, mode, is_active, try_update):
    """
    non-forced create or update enrollment internal function
    """
    try:
        enrollment = api._data_api().create_course_enrollment(username, course_id, mode, is_active)
    except CourseEnrollmentExistsError as err:
        if try_update:
            enrollment = api._data_api().update_course_enrollment(username, course_id, mode, is_active)
        else:
            raise APIException(repr(err) + ", use force to update the existing enrollment") from err
    return enrollment


def _force_create_enrollment(username, course_id, mode, is_active):
    """
    forced create enrollment internal function
    """
    try:
        course_key = get_valid_course_key(course_id)
        user = User.objects.get(username=username)
        enrollment = CourseEnrollment.enroll(user, course_key, check_access=False)
        api._data_api()._update_enrollment(enrollment, is_active=is_active, mode=mode)
    except Exception as err:  # pylint: disable=broad-except
        raise APIException(repr(err)) from err
    return enrollment



""" urls.py """

from django.urls import include, re_path

from eox_core import views
from eox_core.api_schema import docs_ui_view

app_name = 'eox_core'  # pylint: disable=invalid-name

urlpatterns = [  # pylint: disable=invalid-name
    re_path(r'^eox-info$', views.info_view),
    re_path(r'^api/', include('eox_core.api.urls', namespace='eox-api')),
    re_path(r'^data-api/', include('eox_core.api.data.v1.urls', namespace='eox-data-api')),
    re_path(r'^api-docs/$', docs_ui_view, name='apidocs-ui'),
    re_path(r'^tasks-api/', include('eox_core.api.task_dispatcher.v1.urls', namespace='eox-task-api')),
    re_path(r'^support-api/', include('eox_core.api.support.urls', namespace='eox-support-api')),
]



#!/usr/bin/python
"""
Test module for the custom Middlewares
"""
import mock
from django.contrib.auth.models import AnonymousUser
from django.http import Http404
from django.test import RequestFactory, TestCase

from eox_core.middleware import PathRedirectionMiddleware, RedirectionsMiddleware
from eox_core.models import Redirection


class PathRedirectionMiddlewareTest(TestCase):
    """
    Testing the middleware PathRedirectionMiddleware
    """
    def setUp(self):
        """ setup """
        self.request_factory = RequestFactory()
        self.middleware_instance = PathRedirectionMiddleware(get_response=lambda req: None)

    @mock.patch('eox_core.middleware.PathRedirectionMiddleware.process_mktg_redirect')
    @mock.patch('eox_core.middleware.PathRedirectionMiddleware.process_custom_path_redirect')
    @mock.patch('eox_core.middleware.configuration_helper')
    def test_no_redirection_set(self, conf_help_mock,
                                m_process_custom_path_redirect,
                                m_process_mktg_redirect):
        """
        Test the middleware is not redirecting because EDNX_CUSTOM_PATH_REDIRECTS
        and MKTG_REDIRECTS setting are not set in conf helpers
        """
        request = self.request_factory.get('/custom/path/')
        conf_help_mock.has_override_value.return_value = None
        result = self.middleware_instance.process_request(request)
        conf_help_mock.has_override_value.assert_called()
        m_process_custom_path_redirect.assert_not_called()
        m_process_mktg_redirect.assert_not_called()
        self.assertIsNone(result)

    @mock.patch('eox_core.middleware.configuration_helper')
    def test_empty_redirection_custom(self, conf_help_mock):
        """
        Test the middleware is not redirecting because EDNX_CUSTOM_PATH_REDIRECTS
        setting is set to an empty value.
        """
        request = self.request_factory.get('/custom/path/')
        conf_help_mock.get_value.return_value = {}
        result = self.middleware_instance.process_custom_path_redirect(request)
        conf_help_mock.get_value.assert_called_once()
        self.assertIsNone(result)

    @mock.patch('eox_core.middleware.configuration_helper')
    def test_empty_redirection_mktg(self, conf_help_mock):
        """
        Test the middleware is not redirecting because MKTG_REDIRECTS
        setting is set to an empty value.
        """
        request = self.request_factory.get('/custom/path/')
        conf_help_mock.get_value.return_value = {}
        result = self.middleware_instance.process_mktg_redirect(request)
        conf_help_mock.get_value.assert_called_once()
        self.assertIsNone(result)

    @mock.patch('eox_core.middleware.configuration_helper')
    def test_redirection_not_found(self, conf_help_mock):
        """
        Test if a request with a path associated to the not found value is raising
        a 404 response.
        """
        request = self.request_factory.get('/custom/path/')

        conf_help_mock.has_override_value.return_value = True

        redirection = {
            '/custom/path/': 'not_found'
        }
        conf_help_mock.get_value.return_value = redirection

        try:
            result = self.middleware_instance.process_request(request)
        except Http404:
            pass
        else:
            self.assertEqual(result.status_code, 404)

    @mock.patch('eox_core.middleware.configuration_helper')
    def test_redirection_login_required(self, conf_help_mock):
        """
        Test if a request with a path associated to the login required value is
        being redirected to the /login.
        """

        def has_override_value(value):
            """ Mock configuration helper response. """
            return value == "EDNX_CUSTOM_PATH_REDIRECTS"

        request = self.request_factory.get('/custom/path/')
        request.user = AnonymousUser()
        conf_help_mock.has_override_value.side_effect = has_override_value
        conf_help_mock.get_dict.return_value = {'ednx_custom_login_link': '/login'}
        redirection = {
            '/custom/path/': 'login_required'
        }
        conf_help_mock.get_value.return_value = redirection
        result = self.middleware_instance.process_request(request)
        self.assertIn('/login', result.url)

    @mock.patch('eox_core.middleware.configuration_helper')
    def test_mktg_redirection(self, conf_help_mock):
        """
        Test if a request with a path associated to the mktg redirects is
        being redirected to the correct url.
        """
        def has_override_value(value):
            """ Mock configuration helper response. """
            return value == "MKTG_REDIRECTS"

        request = self.request_factory.get('/tos')

        conf_help_mock.has_override_value.return_value = has_override_value
        target_url = '/host.domain/terms'
        conf_help_mock.get_value.return_value = {'tos.html': target_url}
        result = self.middleware_instance.process_request(request)
        self.assertIn(target_url, result.url)


class RedirectionMiddlewareTest(TestCase):
    """
    Testing the middleware RedirectionsMiddleware.
    """
    def setUp(self):
        """ setup """
        self.request_factory = RequestFactory()
        self.middleware_instance = RedirectionsMiddleware(get_response=lambda req: None)

    def test_disabled_feature(self):
        """
        Test the usage of the feature flag
        """
        request = self.request_factory.get('/')
        with mock.patch.dict('django.conf.settings.FEATURES', {'USE_REDIRECTION_MIDDLEWARE': False}):
            result = self.middleware_instance.process_request(request)
        self.assertIsNone(result)

    @mock.patch('eox_core.models.Redirection.objects.get')
    def test_domain_without_redirection(self, redirection_get_mock):
        """
        Test the behaviour when the request domain does not have a redirection object.
        """
        request = self.request_factory.get('/')
        redirection_get_mock.side_effect = Redirection.DoesNotExist  # pylint: disable=no-member
        result = self.middleware_instance.process_request(request)
        redirection_get_mock.assert_called_once()
        self.assertIsNone(result)

    @mock.patch('eox_core.models.Redirection.objects.get')
    def test_redirection(self, redirection_get_mock):
        """
        Test the behaviour when the request domain does not have a redirection object.
        """
        request = self.request_factory.get('/', HTTP_HOST='www.example.com')
        redirection = Redirection("www.example.com", 'example.com', 302, 'http')
        redirection_get_mock.return_value = redirection
        result = self.middleware_instance.process_request(request)

        self.assertIsNotNone(result)



"""
API v1 serializers.
"""
# pylint: disable=abstract-method
from __future__ import absolute_import, unicode_literals

from collections import OrderedDict

from django.conf import settings
from django_countries.serializer_fields import CountryField
from rest_framework import serializers
from rest_framework.fields import HiddenField

from eox_core.edxapp_wrapper.coursekey import get_valid_course_key, validate_org
from eox_core.edxapp_wrapper.enrollments import check_edxapp_enrollment_is_valid
from eox_core.edxapp_wrapper.users import (
    check_edxapp_account_conflicts,
    get_user_read_only_serializer,
    get_user_signup_source,
    get_username_max_length,
)
from eox_core.utils import (
    create_user_profile,
    get_gender_choices,
    get_level_of_education_choices,
    get_registration_extra_fields,
    get_valid_years,
    set_custom_field_restrictions,
    set_select_custom_field,
)

UserSignupSource = get_user_signup_source()  # pylint: disable=invalid-name

MAX_SIGNUP_SOURCES_ALLOWED = 1

USERNAME_MAX_LENGTH = get_username_max_length()

ALLOWED_TYPES = ["text", "email", "select", "textarea", "checkbox", "plaintext", "password", "hidden"]

YEAR_OF_BIRTH_CHOICES = [(str(year), str(year)) for year in get_valid_years()]


class EdxappWithWarningSerializer(serializers.Serializer):
    """
    Mixin serializer to add a warning field to Edxapp serializers
    """
    def __init__(self, *args, **kwargs):
        """
        Conditionally adds a warning field if a context is passed
        """
        super().__init__(*args, **kwargs)

        # This field should only exist if a context is actually passed to the serializer
        if self.context:
            self.fields['warning'] = serializers.SerializerMethodField(read_only=True)

    warning = serializers.CharField(read_only=True, required=False)

    def get_warning(self, obj):
        """Set the warning from the context"""
        if self.context:
            return self.context
        return None


class EdxappUserSerializer(serializers.Serializer):
    """
    Handles the serialization of the user data required to create an edxapp user
    on different backends
    """

    email = serializers.EmailField()
    username = serializers.CharField(max_length=USERNAME_MAX_LENGTH)
    password = serializers.CharField(
        style={'input_type': 'password'},
        write_only=True,
    )
    fullname = serializers.CharField(max_length=255, write_only=True)  # write_only so the user object does not complain for having the name at get_full_name()

    # Extra info to be returned by the api after creating the user
    is_active = serializers.BooleanField(read_only=True)
    is_staff = serializers.BooleanField(read_only=True)
    is_superuser = serializers.BooleanField(read_only=True)

    def validate(self, attrs):
        """
        Check that there are no conflicts on the accounts
        """
        email = attrs.get("email")
        username = attrs.get("username")
        conflicts = check_edxapp_account_conflicts(email, username)
        if conflicts:
            raise serializers.ValidationError(f"Account already exists with the provided: {', '.join(conflicts)}")
        return attrs


class EdxappExtendedUserSerializer(EdxappUserSerializer):
    """
    This serializer allows admiting all the
    UserProfile fields and also the extra fields for the
    userProfile.meta values that are specified in the
    microsite settings.

    If "terms_of_service" or "honor_code" are not marked
    as hidden fields in the REGISTRATION_EXTRA_FIELDS setting,
    then they are added as hidden fields in the serializer
    with the default value set to "true", so these terms
    and conditions are "accepted" when using the create-user
    endpoint.
    """
    year_of_birth = serializers.ChoiceField(choices=YEAR_OF_BIRTH_CHOICES)
    gender = serializers.ChoiceField(choices=get_gender_choices())
    city = serializers.CharField()
    goals = serializers.CharField()
    bio = serializers.CharField(max_length=3000)
    profile_image_uploaded_at = serializers.DateTimeField()
    phone_number = serializers.CharField(max_length=50)
    mailing_address = serializers.CharField()
    courseware = serializers.CharField(max_length=255)
    level_of_education = serializers.ChoiceField(choices=get_level_of_education_choices())
    country = CountryField(required=False)
    terms_of_service = serializers.HiddenField(default='true')
    honor_code = serializers.HiddenField(default='true')

    def __init__(self, *args, **kwargs):  # pylint: disable=too-many-locals
        """
        Add the custom registration fields specified in the settings.
        """
        super().__init__(*args, **kwargs)

        extended_profile_fields = getattr(settings, "extended_profile_fields", [])
        extra_fields = get_registration_extra_fields()
        ednx_custom_registration_fields = getattr(settings, "EDNX_CUSTOM_REGISTRATION_FIELDS", [])
        all_fields = set(self.fields)
        # Obtain only the fields defined in the EdxappExtendedUserSerializer
        non_profile_fields = set(EdxappUserSerializer().fields)
        non_profile_fields.update(["activate_user", "skip_password"])
        profile_fields = all_fields - non_profile_fields

        # Delete the profile fields that are not allowed or are redefined in the ednx_custom_registration setting
        # In case the field IS allowed, check if is required or not
        for field in profile_fields:
            if field not in extra_fields or field in extended_profile_fields:
                self.fields.pop(field)
            else:
                # Hidden fields take their value from the default, so we should not alter the "required" attribute.
                if not isinstance(self.fields[field], HiddenField):
                    self.fields[field].required = extra_fields.get(field) == "required"

        # Adding fields that go inside the UserProfile.meta
        for custom_field in ednx_custom_registration_fields:
            field_name = custom_field.get("name")
            field_type = custom_field.get("type")

            if field_type in ALLOWED_TYPES:
                serializer_field = {}

                serializer_field["required"] = extra_fields.get(field_name) == "required"

                # Check this first since fields of type 'text' are the only ones that can have restrictions
                if field_type == "text":
                    serializer_field = set_custom_field_restrictions(custom_field, serializer_field)

                # Now we add the field to the serializer according to the custom field type defined in the settings
                if field_type == "select":
                    self.fields[field_name] = serializers.ChoiceField(**set_select_custom_field(custom_field, serializer_field))

                elif field_type == "checkbox":
                    self.fields[field_name] = serializers.BooleanField(**serializer_field)

                else:
                    self.fields[field_name] = serializers.CharField(**serializer_field)


class WrittableEdxappUserSerializer(EdxappExtendedUserSerializer):
    """
    Handles the serialization of the user data required to update an edxapp user.
    """
    password = serializers.CharField(
        style={'input_type': 'password'},
    )
    is_active = serializers.BooleanField()

    def validate(self, attrs):
        """
        When an update is being made, then it checks that the user:
            - Is not staff or superuser.
            - Has just one signup source.
            - The field being updated is a safe field.
        """
        # If at least one of these conditions is true, then the user can't be updated.
        safe_fields = getattr(settings, "EOX_CORE_USER_UPDATE_SAFE_FIELDS", [])

        for attr in attrs:
            if attr not in safe_fields:
                raise serializers.ValidationError({"detail": f"You are not allowed to update {attr}."})

        if self.instance.is_staff or self.instance.is_superuser:
            raise serializers.ValidationError({"detail": "You can't update users with roles like staff or superuser."})

        if UserSignupSource.objects.filter(user__email=self.instance.email).count() > MAX_SIGNUP_SOURCES_ALLOWED:
            raise serializers.ValidationError({"detail": "You can't update users with more than one sign up source."})

        return attrs

    def update(self, instance, validated_data):
        """
        Update method for safe fields.
        """
        extended_profile_fields = getattr(settings, 'extended_profile_fields', [])
        # Obtain only the User profile fields defined in the EdxappExtendedUserSerializer
        all_fields = set(EdxappExtendedUserSerializer().fields)
        non_profile_fields = set(EdxappUserSerializer().fields)
        extra_registration_fields = all_fields - non_profile_fields
        # Check if the User has a Profile
        has_profile = hasattr(instance, 'profile')

        extra_registration_fields.update(extended_profile_fields)
        if has_profile:
            profile_meta = instance.profile.get_meta()

        for key, value in validated_data.items():

            if key == "password":
                instance.set_password(value)

            elif key in extra_registration_fields or key == "fullname":
                if not has_profile:
                    create_user_profile(instance)
                    profile_meta = {}
                    has_profile = True

                # First check if the field belongs to the meta
                if key in extended_profile_fields:
                    profile_meta[key] = value

                # Key is one of the user profile fields
                else:
                    if key == "year_of_birth":
                        value = int(value)
                    # 'name' field is sent as 'fullname' in the request
                    if key == "fullname":
                        key = "name"
                    setattr(instance.profile, key, value)

            else:
                setattr(instance, key, value)

        if validated_data:
            if has_profile:
                # Update user profile meta
                instance.profile.set_meta(profile_meta)
                instance.profile.save()
            instance.save()

        return instance


class EdxappUserQuerySerializer(EdxappExtendedUserSerializer):
    """
    Handles the serialization of the context data required to create an edxapp user
    on different backends
    """

    activate_user = serializers.BooleanField(default=False)  # We need to allow the api to activate users later on
    skip_password = serializers.BooleanField(default=False, write_only=True)

    def __init__(self, *args, **kwargs):  # pylint: disable=too-many-locals
        """
        Check skip_password flag to see if password
        should be omitted.
        """
        super().__init__(*args, **kwargs)

        initial_data = getattr(self, "initial_data", {})

        if initial_data.get("skip_password", False):  # pylint: disable=no-member
            self.fields.pop("password", None)


class EdxappEnrollmentAttributeSerializer(serializers.Serializer):
    """
    Attributes serializer
    """
    namespace = serializers.CharField()
    name = serializers.CharField()
    value = serializers.CharField()


class EdxappValidatedCourseIDField(serializers.Field):
    """
    CourseKey Field
    """
    def to_representation(self, value):
        return str(value)

    def to_internal_value(self, data):
        """
        Check course_id has the correct format and that it is allowed inside the org
        """
        if validate_org(data):  # pylint: disable=no-else-return
            return str(get_valid_course_key(data))
        else:
            raise serializers.ValidationError(f'Invalid course_id {data}')


class EdxappCourseEnrollmentSerializer(serializers.Serializer):
    """Serializes CourseEnrollment

    Aggregates all data from the Course Enrollment table, and pulls in the serialization for
    the Course Descriptor and course modes, to give a complete representation of course enrollment.

    """

    username = serializers.CharField(max_length=USERNAME_MAX_LENGTH, default=None, source='user')
    is_active = serializers.BooleanField(default=True)
    mode = serializers.CharField(max_length=100)
    enrollment_attributes = EdxappEnrollmentAttributeSerializer(many=True, required=False)
    course_id = EdxappValidatedCourseIDField()
    created = serializers.DateTimeField(read_only=True)

    def validate(self, attrs):
        """
        Check that there are no issues with enrollment
        """
        errors = check_edxapp_enrollment_is_valid(**attrs)
        if errors:
            raise serializers.ValidationError(", ".join(errors))
        return attrs

    class Meta:
        """
        Add extra details for swagger
        """
        swagger_schema_fields = {
            "example": OrderedDict(
                [
                    ("username", "johndoe"),
                    ("is_active", True),
                    ("mode", "audit"),
                    ("enrollment_attributes", []),
                    ("course_id", "course-v1:edX+DemoX+Demo_Course"),
                    ("created", "2021-10-11T10:06:17.876505-05:00"),
                ],
            ),
        }


class EdxappCourseEnrollmentQuerySerializer(EdxappCourseEnrollmentSerializer):
    """
    Handles the serialization of the context data required to create an enrollment
    on different backends
    """
    username = serializers.CharField(max_length=USERNAME_MAX_LENGTH, default=None)
    email = serializers.CharField(max_length=255, default=None)
    force = serializers.BooleanField(default=False)
    course_id = EdxappValidatedCourseIDField(default=None)
    bundle_id = serializers.CharField(max_length=255, default=None)

    class Meta(EdxappCourseEnrollmentSerializer.Meta):
        """
        Add extra details for swagger
        """
        swagger_schema_fields = {
            "example": OrderedDict(
                {
                    "force": False,
                    **EdxappCourseEnrollmentSerializer.Meta.swagger_schema_fields.get("example")
                },
            ),
        }


class EdxappCoursePreEnrollmentSerializer(EdxappWithWarningSerializer):
    """Serialize CourseEnrollmentAllowed

    Handles the serialization of the context data required to create a course whitelisting or pre-enrollments for a user
    """
    course_id = EdxappValidatedCourseIDField()
    auto_enroll = serializers.BooleanField(default=True)
    email = serializers.EmailField()


def EdxappUserReadOnlySerializer(*args, **kwargs):   # pylint: disable=invalid-name
    """
    Fake class to lazily retrieve UserReadOnlySerializer
    """
    return get_user_read_only_serializer()(*args, **kwargs)


class EdxappSectionBreakdownSerializer(serializers.Serializer):
    """
    Serializes the `section_breakdown` portion of the Grades API.
    """
    attempted = serializers.BooleanField()
    assignment_type = serializers.CharField()
    percent = serializers.FloatField()
    score_earned = serializers.FloatField()
    score_possible = serializers.FloatField()
    subsection_name = serializers.CharField()


class EdxappGradingRawPolicySerializer(serializers.Serializer):
    """
    Serializes the items in the GRADER part of grading policy.
    """
    assignment_type = serializers.CharField(source='type')
    count = serializers.IntegerField(source='min_count')
    dropped = serializers.IntegerField(source='drop_count')
    weight = serializers.FloatField()


class EdxappGradingPolicySerializer(serializers.Serializer):
    """
    Serializes the course grading policy
    """
    grader = EdxappGradingRawPolicySerializer(many=True, source="GRADER")
    grade_cutoffs = serializers.DictField(child=serializers.FloatField(), source="GRADE_CUTOFFS")


class EdxappGradeSerializer(serializers.Serializer):
    """
    Serializes the grades data for a user in a given course
    """
    earned_grade = serializers.FloatField()
    grading_policy = EdxappGradingPolicySerializer(required=False)
    section_breakdown = EdxappSectionBreakdownSerializer(many=True, required=False)

    class Meta:
        """
        Add extra details for swagger
        """
        swagger_schema_fields = {
            "example":
            {
                "earned_grade": 0.5,
                "grading_policy": {
                    "grader": [
                        {
                            "assignment_type": "Homework",
                            "count": 1,
                            "dropped": 0,
                            "weight": 1.0
                        }
                    ],
                    "grade_cutoffs": {
                        "Pass": 0.5
                    }
                },
                "section_breakdown": [
                    {
                        "attempted": True,
                        "assignment_type": "Homework",
                        "percent": 0.5,
                        "score_earned": 5,
                        "score_possible": 10,
                        "subsection_name": "Homework - Questions"
                    }
                ]
            }
        }



"""
The logging module defines functions that are used for the logging of the
pipeline steps.
"""
import logging
import sys
from pprint import pformat

LOG = logging.getLogger(__name__)


# pylint: disable=unused-argument,keyword-arg-before-vararg
def logging_pipeline_step(level, log_message, **local_vars):
    """
    This function logs an info or error level message with the information of the pipeline step.
    If the logLevel setting is in DEBUG mode then it logs all the arguments passed to the step, if not
    it will only show the name of the step, user, backend name, redirect_uri and a custom message.

    For example:
        To set the verbosity level to DEBUG add the setting:

        "BACKEND_OPTIONS": { "logLevel":"DEBUG" }

    Arguments:
        - level: This is used to describe the severity of the message that the django logger will handle.
                If "error" is passed, then an ERROR level logging message will be displayed. Otherwise,
                an INFO log will be displayed.
        - log_message: Custom message to be shown in the log.
        - local_vars: local variables, this must always be passed as **locals().
    """
    extra_info = {}
    user = local_vars.get("user")
    backend = local_vars.get("backend")

    step = sys._getframe(1).f_code.co_name  # pylint: disable=protected-access
    backend_name = getattr(backend, "name", "")
    uri = getattr(backend, "redirect_uri", "")
    msg = log_message

    message = f"PIPELINE-STEP:{step} - USER:{user} - BACKEND:{backend_name} - REDIRECT_URI:{uri} - MESSAGE:{msg}"

    if backend and backend.setting("BACKEND_OPTIONS", {}).get("logLevel", "").lower() == "debug":
        details = local_vars.get("details")
        args = local_vars.get("args", [])
        kwargs = local_vars.get("kwargs", {})
        response = kwargs.pop("response", None)
        # We do not want to show the id_token in the logs
        if response:
            response.pop("id_token", None)

        extra_info["details"] = pformat(details)
        extra_info["pipeline_step_args"] = pformat(args)
        extra_info["request"] = pformat(kwargs.pop("request", None))
        extra_info["response"] = pformat(response)
        extra_info["kwargs"] = pformat(kwargs)

    if level.lower() == "error":
        LOG.exception(message, extra=extra_info)
    else:
        LOG.info(message, extra=extra_info)



"""
URLs for the Microsite API
"""
from django.urls import include, re_path

from .routers import ROUTER
from .views import CeleryTasksStatus

app_name = 'eox_core'  # pylint: disable=invalid-name

urlpatterns = [  # pylint: disable=invalid-name
    re_path(r'^v1/', include((ROUTER.urls, 'eox_core'), namespace='eox-data-api-v1')),
    re_path(r'^v1/tasks/(?P<task_id>.*)$', CeleryTasksStatus.as_view(), name="celery-data-api-tasks"),
]



"""
Courses definitions.
"""

from importlib import import_module

from django.conf import settings


def get_courses_accessible_to_user(*args, **kwargs):
    """ Gets the _courses_accessible_to_user function. """

    backend_function = settings.EOX_CORE_COURSES_BACKEND
    backend = import_module(backend_function)

    return backend.courses_accessible_to_user(*args, **kwargs)


def get_process_courses_list(*args, **kwargs):
    """ Gets the _process_courses_list function. """

    backend_function = settings.EOX_CORE_COURSES_BACKEND
    backend = import_module(backend_function)

    return backend.get_process_courses_list(*args, **kwargs)


def get_course_details_fields():
    """ Gets course details fields. """

    backend_function = settings.EOX_CORE_COURSES_BACKEND
    backend = import_module(backend_function)

    return backend.get_course_details_fields()


def get_first_course_key():
    """ Gets the first course key string. """

    backend_function = settings.EOX_CORE_COURSES_BACKEND
    backend = import_module(backend_function)

    return backend.get_first_course_key()


def get_course_overview():
    """ Gets the course overview model from edxapp. """

    backend_function = settings.EOX_CORE_COURSES_BACKEND
    backend = import_module(backend_function)

    return backend.get_course_overview()



"""
Settings for eox-core
"""

from __future__ import absolute_import, unicode_literals

from .common import *  # pylint: disable=wildcard-import, unused-wildcard-import


class SettingsClass:
    """ dummy settings class """


def plugin_settings(settings):  # pylint: disable=function-redefined
    """
    Defines eox-core settings when app is used as a plugin to edx-platform.
    See: https://github.com/edx/edx-platform/blob/master/openedx/core/djangoapps/plugins/README.rst
    """
    settings.EOX_CORE_USERS_BACKEND = "eox_core.edxapp_wrapper.backends.users_m_v1_test"
    settings.EOX_CORE_ENROLLMENT_BACKEND = "eox_core.edxapp_wrapper.backends.enrollment_l_v1"
    settings.EOX_CORE_PRE_ENROLLMENT_BACKEND = "eox_core.edxapp_wrapper.backends.pre_enrollment_l_v1"
    settings.EOX_CORE_COURSEKEY_BACKEND = "eox_core.edxapp_wrapper.backends.coursekey_m_v1"
    settings.EOX_CORE_CERTIFICATES_BACKEND = "eox_core.edxapp_wrapper.backends.certificates_h_v1_test"
    settings.EOX_CORE_CONFIGURATION_HELPER_BACKEND = "eox_core.edxapp_wrapper.backends.configuration_helpers_h_v1_test"
    settings.EOX_CORE_COURSEWARE_BACKEND = "eox_core.edxapp_wrapper.backends.courseware_h_v1"
    settings.EOX_CORE_GRADES_BACKEND = "eox_core.edxapp_wrapper.backends.grades_h_v1"
    settings.EOX_CORE_MICROSITES_BACKEND = "eox_core.edxapp_wrapper.backends.microsite_configuration_h_v1"
    settings.EOX_CORE_STORAGES_BACKEND = "eox_core.edxapp_wrapper.backends.storages_i_v1_test"
    settings.EOX_CORE_LOAD_PERMISSIONS = False
    settings.DATA_API_DEF_PAGE_SIZE = 1000
    settings.DATA_API_MAX_PAGE_SIZE = 5000
    settings.EOX_CORE_ENABLE_UPDATE_USERS = True
    settings.EOX_CORE_USER_UPDATE_SAFE_FIELDS = ["is_active", "password", "fullname"]
    settings.EOX_CORE_BEARER_AUTHENTICATION = 'eox_core.edxapp_wrapper.backends.bearer_authentication_j_v1_test'
    settings.EOX_CORE_THIRD_PARTY_AUTH_BACKEND = 'eox_core.edxapp_wrapper.backends.third_party_auth_l_v1'


SETTINGS = SettingsClass()
plugin_settings(SETTINGS)
vars().update(SETTINGS.__dict__)

try:
    import edx_when  # pylint: disable=unused-import
    INSTALLED_APPS += ('eox_core', 'edx_when.apps.EdxWhenConfig')
except ImportError:
    INSTALLED_APPS += ('eox_core',)

ROOT_URLCONF = 'eox_core.urls'
ALLOWED_HOSTS = ['*']

# This key needs to be defined so that the check_apps_ready passes and the
# AppRegistry is loaded
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'db.sqlite3',
    }
}

ENV_ROOT = '.'

FEATURES = {}
FEATURES['USE_REDIRECTION_MIDDLEWARE'] = True

COURSE_KEY_PATTERN = r'(?P<course_key_string>[^/+]+(/|\+)[^/+]+(/|\+)[^/?]+)'
COURSE_ID_PATTERN = COURSE_KEY_PATTERN.replace('course_key_string', 'course_id')

USERNAME_REGEX_PARTIAL = r'[\w .@_+-]+'
USERNAME_PATTERN = fr'(?P<username>{USERNAME_REGEX_PARTIAL})'

PROCTORING_SETTINGS = {
    'LINK_URLS': {
        'contact_us': 'http://test-link.com/contact',
        'faq': 'http://test-link.com/faq',
        'online_proctoring_rules': 'http://test-link.com/rules',
        'tech_requirements': 'http://test-link.com/tech',
    },
    'SITE_NAME': 'test-site-name',
    'PLATFORM_NAME': 'test-platform-name',
}

PROCTORING_BACKENDS = {
    'software_secure': {
        'crypto_key': 'test-key',
        'exam_register_endpoint': 'test-enpoint',
        'exam_sponsor': 'test-sponsor',
        'organization': 'test-org',
        'secret_key': 'test-secret-key',
        'secret_key_id': 'test-secret-key-id',
        'software_download_url': 'http://test.com/url',
    },
    'DEFAULT': 'software_secure',
}



"""
Models used in eox-core.
"""
from django.db import models


class Redirection(models.Model):
    """
    This object stores the redirects for a domain
    """

    HTTP = 'http'
    HTTPS = 'https'

    SCHEME = (
        (HTTP, 'http'),
        (HTTPS, 'https'),
    )

    STATUS = (
        (301, 'Temporary'),
        (302, 'Permanent'),
    )

    domain = models.CharField(max_length=253, db_index=True,
                              help_text='use only the domain name, e.g. cursos.edunext.co')
    target = models.CharField(max_length=253)
    scheme = models.CharField(max_length=5, choices=SCHEME, default=HTTP)
    status = models.IntegerField(choices=STATUS, default=301)

    class Meta:
        """
        Model meta class.
        """
        # Note to ops: The table already exists under a different name due to the migration from EOE.
        db_table = 'edunext_redirection'

    def __unicode__(self):
        return f"Redirection from {self.domain} to {self.target}. Protocol {self.scheme}. Status {self.status}"



# -*- coding: utf-8 -*-
""" Configuration as explained on tutorial
github.com/edx/edx-platform/tree/master/openedx/core/djangoapps/plugins"""
from __future__ import unicode_literals

from django.apps import AppConfig


class EoxCoreConfig(AppConfig):
    """App configuration"""
    name = 'eox_core'
    verbose_name = "eduNEXT Openedx Extensions"

    plugin_app = {
        'url_config': {
            'lms.djangoapp': {
                'namespace': 'eox-core',
                'regex': r'^eox-core/',
                'relative_path': 'urls',
            }
        },
        'settings_config': {
            'lms.djangoapp': {
                'test': {'relative_path': 'settings.test'},
                'common': {'relative_path': 'settings.common'},
                'production': {'relative_path': 'settings.production'},
                'devstack': {'relative_path': 'settings.devstack'},
            },
        },
    }


class EoxCoreCMSConfig(EoxCoreConfig):
    """App configuration"""
    name = 'eox_core'
    verbose_name = "eduNEXT Openedx Extensions"

    plugin_app = {
        'url_config': {
            'cms.djangoapp': {
                'namespace': 'eox-core',
                'regex': r'^eox-core/',
                'relative_path': 'urls',
            }
        },
        'settings_config': {
            'cms.djangoapp': {
                'test': {'relative_path': 'settings.test'},
                'common': {'relative_path': 'settings.common'},
                'production': {'relative_path': 'settings.production'},
            },
        },
    }



""" Tests for public user creation API. """
from __future__ import absolute_import, unicode_literals

import mock
from django.conf import settings
from django.test import TestCase

from ..users import create_edxapp_user


class CreateEdxappUserTest(TestCase):
    """ Tests for the public API module """

    @mock.patch('eox_core.edxapp_wrapper.users.import_module')
    def test_import_the_backend(self, m_import):
        """ Test we import the correct backend defined in the settings """

        create_edxapp_user()
        m_import.assert_called_with(settings.EOX_CORE_USERS_BACKEND)

    @mock.patch('eox_core.edxapp_wrapper.users.import_module')
    def test_call_the_backend(self, m_import):
        """ Test we use the imported backend """
        m_user_backend = mock.MagicMock()
        m_import.return_value = m_user_backend

        data = {
            "email": "something",
            "username": "something",
        }

        create_edxapp_user(data)
        m_user_backend.create_edxapp_user.assert_called_with(data)



# -*- coding: utf-8 -*-
""" Admin.py """
from __future__ import unicode_literals

from django.contrib import admin

from eox_core.models import Redirection


class RedirectionAdmin(admin.ModelAdmin):
    """
    Admin view to see and edit edunext redirection objects.
    """
    list_display = [
        'target',
        'domain',
        'scheme',
    ]
    search_fields = ('target', 'domain',)


admin.site.register(Redirection, RedirectionAdmin)



"""
Third Party Auth Exception Middleware definitions.
"""

from importlib import import_module

from django.conf import settings


def get_tpa_exception_middleware():
    """Get the ExceptionMiddleware class."""
    backend_function = settings.EOX_CORE_THIRD_PARTY_AUTH_BACKEND
    backend = import_module(backend_function)
    return backend.get_tpa_exception_middleware()



"""
Courseware definitions.
"""

from importlib import import_module

from django.conf import settings


def get_courseware_courses():
    """ Gets courses. """

    backend_function = settings.EOX_CORE_COURSEWARE_BACKEND
    backend = import_module(backend_function)

    return backend.get_courseware_courses()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Pre-enrollment public function definitions
"""

from importlib import import_module

from django.conf import settings


def create_pre_enrollment(*args, **kwargs):
    """
    Create a pre-enrollment for an existing or future user
    """

    backend_function = settings.EOX_CORE_PRE_ENROLLMENT_BACKEND
    backend = import_module(backend_function)

    return backend.create_pre_enrollment(*args, **kwargs)


def update_pre_enrollment(*args, **kwargs):
    """
    Update a pre-enrollment for an existing or future user
    """

    backend_function = settings.EOX_CORE_PRE_ENROLLMENT_BACKEND
    backend = import_module(backend_function)

    return backend.update_pre_enrollment(*args, **kwargs)


def delete_pre_enrollment(*args, **kwargs):
    """
    Delete a pre-enrollment for an existing or future user
    """

    backend_function = settings.EOX_CORE_PRE_ENROLLMENT_BACKEND
    backend = import_module(backend_function)

    return backend.delete_pre_enrollment(*args, **kwargs)


def get_pre_enrollment(*args, **kwargs):
    """
    Get a pre-enrollment for an existing or future user
    """

    backend_function = settings.EOX_CORE_PRE_ENROLLMENT_BACKEND
    backend = import_module(backend_function)

    return backend.get_pre_enrollment(*args, **kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Users public function definitions
"""

from importlib import import_module

from django.conf import settings


def create_enrollment(*args, **kwargs):
    """ Creates the edxapp user """

    backend_function = settings.EOX_CORE_ENROLLMENT_BACKEND
    backend = import_module(backend_function)

    return backend.create_enrollment(*args, **kwargs)


def update_enrollment(*args, **kwargs):
    """ Update enrollments on edxapp """

    backend_function = settings.EOX_CORE_ENROLLMENT_BACKEND
    backend = import_module(backend_function)

    return backend.update_enrollment(*args, **kwargs)


def get_enrollment(*args, **kwargs):
    """ Get enrollments on edxapp """

    backend_function = settings.EOX_CORE_ENROLLMENT_BACKEND
    backend = import_module(backend_function)

    return backend.get_enrollment(*args, **kwargs)


def delete_enrollment(*args, **kwargs):
    """ Delete enrollments on edxapp """

    backend_function = settings.EOX_CORE_ENROLLMENT_BACKEND
    backend = import_module(backend_function)

    return backend.delete_enrollment(*args, **kwargs)


# pylint: disable=invalid-name
def check_edxapp_enrollment_is_valid(*args, **kwargs):
    """ Checks the db for accounts with the same email or password """

    backend_function = settings.EOX_CORE_ENROLLMENT_BACKEND
    backend = import_module(backend_function)

    return backend.check_edxapp_enrollment_is_valid(*args, **kwargs)



"""
Task dispatcher api urls
"""
from django.urls import re_path

from eox_core.api.task_dispatcher.v1.views import TaskAPI

app_name = 'eox_core'  # pylint: disable=invalid-name

urlpatterns = [
    re_path(r'^v1/tasks/$', TaskAPI.as_view(), name='task-api'),
]



#!/usr/bin/python
"""
Test module for RedirectionModel
"""
from django.core.exceptions import ValidationError
from django.test import TestCase

from eox_core.models import Redirection


class RedirectionModelTest(TestCase):
    """
    Test the model where most of the logic is
    """

    def test_model_creation(self):
        """
        Answers the question: Can we create a model?
        """
        obj = Redirection()
        obj.domain = "localhost"
        obj.scheme = "https"
        obj.status = "302"
        obj.target = "example.com/path"

        obj.full_clean()

    def test_model_creation_fail(self):
        """
        Answers the question: If we make a wrong object, does it complain?
        """
        obj = Redirection()
        obj.domain = "localhost"
        with self.assertRaises(ValidationError):
            obj.full_clean()



# -*- coding: utf-8 -*-
""" . """
from django.contrib.auth.models import User
from django.test import TestCase
from mock import patch
from rest_framework.test import APIClient


class TestEnrollmentsAPI(TestCase):
    """ Tests for the enrollments endpoints """

    patch_permissions = patch('eox_core.api.v1.permissions.EoxCoreAPIPermission.has_permission', return_value=True)

    def setUp(self):
        """ setup """
        super().setUp()
        self.api_user = User(1, 'test@example.com', 'test')
        self.client = APIClient()
        self.client.force_authenticate(user=self.api_user)

    @patch_permissions
    @patch('eox_core.api.v1.views.get_edxapp_user')
    def test_api_get_validation(self, m_get_user, *_):
        """ Test that the GET method requires some parameters """
        response = self.client.get('/api/v1/enrollment/')

        self.assertEqual(response.status_code, 400)
        self.assertIn('username', response.data[0])

        params = {
            'username': 'test',
        }
        response = self.client.get('/api/v1/enrollment/', params)
        self.assertEqual(response.status_code, 400)
        self.assertIn('course_id', response.data[0])

    @patch('eox_core.api.v1.views.get_enrollment')
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch_permissions
    def test_api_get(self, _, m_get_user, m_get_enrollment):
        """ Test that the GET method works under normal conditions """
        m_get_user.return_value.username = 'testusername'
        m_get_enrollment.return_value = None, None

        params = {
            'course_id': 'course-v1:org+course+run',
            'username': 'testusername',
        }
        response = self.client.get('/api/v1/enrollment/', params)
        self.assertEqual(response.status_code, 200)

        m_get_user.assert_called_once_with(username='testusername')
        m_get_enrollment.assert_called_once_with(
            username='testusername',
            course_id='course-v1:org+course+run',
        )

    @patch('eox_core.api.v1.serializers.check_edxapp_enrollment_is_valid', )
    @patch_permissions
    def test_api_post_validation(self, _, m_check_enrollment):
        """ Test that the POST method validates inputs """
        response = self.client.post('/api/v1/enrollment/')
        self.assertEqual(response.status_code, 400)
        self.assertIn('mode', response.data)

        m_check_enrollment.return_value = ['Email or username needed']
        params = {
            'mode': 'audit',
        }
        response = self.client.post('/api/v1/enrollment/', data=params)
        self.assertEqual(response.status_code, 400)
        self.assertIn('non_field_errors', response.data)

        m_check_enrollment.return_value = ['You have to provide a course_id or bundle_id']
        params = {
            'mode': 'audit',
            'username': 'test',
        }
        response = self.client.post('/api/v1/enrollment/', data=params)
        self.assertEqual(response.status_code, 400)
        self.assertIn('non_field_errors', response.data)

    @patch_permissions
    @patch('eox_core.api.v1.serializers.validate_org')
    @patch('eox_core.api.v1.serializers.get_valid_course_key')
    @patch('eox_core.api.v1.serializers.check_edxapp_enrollment_is_valid', return_value=[])
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.create_enrollment')
    def test_api_post_works(self, m_create_enrollment, *_):
        """ Test that the POST method works in normal conditions """

        m_enrollment = {
            'mode': 'audit',
            'user': 'test',  # this is the source value for the username field in the serializer
            'course_id': 'course-v1:org+course+run',
            'is_active': True,
        }
        m_create_enrollment.return_value = m_enrollment, None
        params = {
            'mode': 'audit',
            'username': 'test',
            'course_id': 'course-v1:org+course+run',
        }
        response = self.client.post('/api/v1/enrollment/', data=params)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, {**response.data, **params})

    @patch_permissions
    @patch('eox_core.api.v1.serializers.validate_org')
    @patch('eox_core.api.v1.serializers.get_valid_course_key')
    @patch('eox_core.api.v1.serializers.check_edxapp_enrollment_is_valid', return_value=[])
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.update_enrollment')
    def test_api_put_works(self, m_update_enrollment, m_get_user, *_):
        """ Test that the PUT method works in normal conditions with a list of enrollments """

        enrollments_response = {
            'mode': 'audit',
            'user': 'test',  # this is the source value for the username field in the serializer
            'course_id': 'course-v1:org+course+run',
            'is_active': True,
        }

        m_update_enrollment.return_value = enrollments_response
        params = [{
            'mode': 'audit',
            'username': 'test',
            'course_id': 'course-v1:org+course+run',
        }, {
            'mode': 'audit',
            'username': 'test',
            'course_id': 'course-v1:org+course_2+run',
        }]

        response = self.client.put('/api/v1/enrollment/', data=params, format='json')
        m_get_user.assert_called()
        self.assertEqual(response.status_code, 200)
        self.assertIn('course_id', response.data[0])
        self.assertIn('is_active', response.data[0])
        self.assertEqual(2, len(response.data))

    @patch_permissions
    @patch('eox_core.api.v1.views.get_edxapp_user')
    @patch('eox_core.api.v1.views.delete_enrollment')
    def test_api_delete_works(self, m_delete_enrollment, m_get_user, *_):
        """ Test that the DELETE method works in normal conditions with a bundle """

        params = {
            'username': 'test',
            'course_id': 'course-v1:org+course+run',
        }
        response = self.client.delete('/api/v1/enrollment/', params)
        m_get_user.assert_called_once_with(username='test')
        m_delete_enrollment.assert_called_once_with(course_id='course-v1:org+course+run', user=m_get_user.return_value)
        self.assertEqual(response.status_code, 204)



# -*- coding: utf-8 -*-
"""The generic views for the exc-core plugin project"""

from __future__ import unicode_literals

import json
from os.path import dirname, realpath
from subprocess import CalledProcessError, check_output

import six
from django.http import HttpResponse

import eox_core


def info_view(request):
    """
    Basic view to show the working version and the exact git commit of the
    installed app
    """
    try:
        working_dir = dirname(realpath(__file__))
        git_data = six.text_type(check_output(
            ["git", "rev-parse", "HEAD"], cwd=working_dir))
    except CalledProcessError:
        git_data = ""

    response_data = {
        "version": eox_core.__version__,
        "name": "eox-core",
        "git": git_data.rstrip('\r\n'),
    }
    return HttpResponse(
        json.dumps(response_data),
        content_type="application/json"
    )



"""
Backend for contenstore courses.
"""

from cms.djangoapps.contentstore.views.course import (  # pylint: disable=import-error
    _process_courses_list,
    get_courses_accessible_to_user,
)
from openedx.core.djangoapps.content.course_overviews.models import CourseOverview  # pylint: disable=import-error
from openedx.core.djangoapps.models.course_details import CourseDetails  # pylint: disable=import-error


def courses_accessible_to_user(*args, **kwargs):
    """ get get_courses_accessible_to_user function. """
    return get_courses_accessible_to_user(*args, **kwargs)


def get_process_courses_list(*args, **kwargs):
    """ get _process_courses_list function. """
    return _process_courses_list(*args, **kwargs)


def get_course_details_fields():
    """ get CourseDetails fields. """
    course_details = CourseDetails(org=None, course_id=None, run=None)
    all_fields = vars(course_details)

    return all_fields.keys()


def get_first_course_key():
    """
    Returns the course key of any course in order to get
    the advance settings keys to populate the course settings
    tab in the Course management view.
    """

    course = CourseOverview.objects.all()[:1]
    if course:
        course_location = course[0].location
    else:
        return ''
    return course_location.course_key


def get_course_overview():
    """
    Gets course overview model from edxapp.
    """
    return CourseOverview



"""Backend for the third party authentication exception middleware."""


def get_tpa_exception_middleware():
    """Get the ExceptionMiddleware class."""
    try:
        from common.djangoapps.third_party_auth.middleware import \
            ExceptionMiddleware  # pylint: disable=import-outside-toplevel
    except ImportError:
        from django.utils.deprecation import \
            MiddlewareMixin as ExceptionMiddleware  # pylint: disable=import-outside-toplevel
    return ExceptionMiddleware



"""
This file implements utils used for sentry integration.

See: https://github.com/eduNEXT/eox-core#integrations-with-third-party-services
"""
import importlib
import logging
import re

import six
from django.conf import settings

LOG = logging.getLogger(__name__)


def load_class(full_class_string):
    """
    dynamically load a class from a string
    """

    class_data = full_class_string.split(".")
    module_path = ".".join(class_data[:-1])
    class_str = class_data[-1]

    module = importlib.import_module(module_path)
    return getattr(module, class_str)


class ExceptionFilterSentry:
    """
    This class is a helper to filter exception events before send them to
    sentry.io service (See: https://github.com/getsentry/sentry-python/issues/149#issuecomment-434448781).
    It relies on the EOX_CORE_SENTRY_IGNORED_ERRORS setting. This setting is a list of rules that defines
    which exceptions to ignore. An example below:

    EOX_CORE_SENTRY_IGNORED_ERRORS = [
        {
            "exc_class": "openedx.core.djangoapps.user_authn.exceptions.AuthFailedError",
            "exc_text": ["AuthFailedError.*Email or password is incorrect"]
        },
    ]
    Every rule support only 2 keys for now:
    - exc_class: the path to the exception class we want to ignore. It can only be one
    - exc_text: a list of regex expressions to search on the last traceback frame text of the exception

    In this example we have only one rule. We are ignoring AuthFailedError exceptions whose traceback text
    has a match with the regex provided in the exc_text unique element. If exc_text contains more than one
    regex, the exception is ignored if any of the regex matches the traceback text.

    NOTE: This implementation supports the legacy configuration where only a list of ignored exception classes
    was specified:
    EOX_CORE_SENTRY_IGNORED_ERRORS = [
        "openedx.core.djangoapps.user_authn.exceptions.AuthFailedError",
        "xmodule.exceptions.NotFoundError",
    ]
    In this mode, all instances of the listed exception classes will be ignored by Sentry
    """
    hint = None
    exc_text = ''
    exc_value = None

    validation_exc_methods = {
        'exc_class': 'validate_exc_class',
        'exc_text': 'validate_exc_text'
    }

    def init_exception_values(self, event, hint):
        """
        Initialization of required values
        """
        self.hint = hint

        if 'log_record' in hint:
            self.exc_text = getattr(hint['log_record'], 'exc_text', '')

        if 'exc_info' in hint:
            _exc_type, exc_value, _tb = hint['exc_info']
            self.exc_value = exc_value
            if not self.exc_text:
                self.exc_text = str(exc_value)

    def clear_exception_values(self):
        """
        Clear values from previous validation
        """
        self.hint = None
        self.exc_text = ''
        self.exc_value = None

    def __call__(self, event, hint):
        """
        Workaround to prevent certain exceptions to be sent to sentry.io
        See: https://github.com/getsentry/sentry-python/issues/149#issuecomment-434448781
        """
        self.clear_exception_values()
        self.init_exception_values(event, hint)

        ignore_event = False
        for rule in settings.EOX_CORE_SENTRY_IGNORED_ERRORS:
            try:
                ignore_event = self.validate_single_ignore_rule(rule)
            except Exception as err:  # pylint: disable=broad-except
                ignore_event = False
                LOG.warning('Could not evaluate Sentry ignore rule %s on event %s. Reason: %s', rule, event, err)

            # If the event meet the conditions to be ignored, break the loop
            if ignore_event:
                return None

        return event

    def validate_single_ignore_rule(self, rule):
        """
        Validates if a given ignored exception rule matches the current exception
        """
        # Adding support for legacy ignored exception classes list in EOX_CORE_SENTRY_IGNORED_ERRORS
        if isinstance(rule, six.string_types):
            rule = {
                "exc_class": rule
            }

        for key, value in six.iteritems(rule):
            # Try to get the validation method based on the current key of the rule
            validate_method = self.validation_exc_methods.get(key, None)
            # If validate_method is not defined, return False
            if not validate_method:
                return False
            result = getattr(self, validate_method)(value)
            if not result:
                return False

        return True

    def validate_exc_class(self, exc_class_path):
        """
        Validate if the current exception object is an instance of the class specified
        on the given path
        """
        exc_class = type(None)
        try:
            exc_class = load_class(exc_class_path)
        except Exception:  # pylint: disable=broad-except
            pass

        return isinstance(self.exc_value, exc_class)

    def validate_exc_text(self, exc_text_expr):
        """
        Validate if any of the given regex matches with the current exception last
        traceback text
        """
        if not isinstance(exc_text_expr, list):
            exc_text_expr = [exc_text_expr]

        for expr in exc_text_expr:
            if re.search(expr, self.exc_text):
                return True

        return False



# -*- coding: utf-8 -*-
# Generated by Django 1.11.21 on 2019-07-15 21:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Redirection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('domain', models.CharField(db_index=True, help_text='use only the domain name, e.g. cursos.edunext.co', max_length=253)),
                ('target', models.CharField(max_length=253)),
                ('scheme', models.CharField(choices=[('http', 'http'), ('https', 'https')], default='http', max_length=5)),
                ('status', models.IntegerField(choices=[(301, 'Temporary'), (302, 'Permanent')], default=301)),
            ],
            options={
                'db_table': 'edunext_redirection',
            },
        ),
    ]



"""
Settings for eox_core project meant to be called on the edx-platform/*/envs/aws.py module
"""
from .common import *  # pylint: disable=wildcard-import, unused-wildcard-import

try:
    import sentry_sdk
    from sentry_sdk.integrations.django import DjangoIntegration
except ImportError:
    sentry_sdk = DjangoIntegration = None


def plugin_settings(settings):  # pylint: disable=function-redefined
    """
    Set of plugin settings used by the Open Edx platform.
    More info: https://github.com/edx/edx-platform/blob/master/openedx/core/djangoapps/plugins/README.rst
    """
    settings.EOX_CORE_COMMENTS_SERVICE_USERS_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_COMMENTS_SERVICE_USERS_BACKEND',
        settings.EOX_CORE_COMMENTS_SERVICE_USERS_BACKEND
    )
    settings.EOX_CORE_BEARER_AUTHENTICATION = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_BEARER_AUTHENTICATION',
        settings.EOX_CORE_BEARER_AUTHENTICATION
    )
    settings.EOX_CORE_USERS_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_USERS_BACKEND',
        settings.EOX_CORE_USERS_BACKEND
    )
    settings.EOX_CORE_ENROLLMENT_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_ENROLLMENT_BACKEND',
        settings.EOX_CORE_ENROLLMENT_BACKEND
    )
    settings.EOX_CORE_PRE_ENROLLMENT_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_PRE_ENROLLMENT_BACKEND',
        settings.EOX_CORE_PRE_ENROLLMENT_BACKEND
    )
    settings.EOX_CORE_CERTIFICATES_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_CERTIFICATES_BACKEND',
        settings.EOX_CORE_CERTIFICATES_BACKEND
    )
    settings.EOX_CORE_COURSEWARE_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_COURSEWARE_BACKEND',
        settings.EOX_CORE_COURSEWARE_BACKEND
    )
    settings.EOX_CORE_GRADES_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_GRADES_BACKEND',
        settings.EOX_CORE_GRADES_BACKEND
    )
    settings.EOX_CORE_STORAGES_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_STORAGES_BACKEND',
        settings.EOX_CORE_STORAGES_BACKEND
    )

    settings.EOX_CORE_ENABLE_STATICFILES_STORAGE = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_ENABLE_STATICFILES_STORAGE',
        settings.EOX_CORE_ENABLE_STATICFILES_STORAGE
    )
    settings.EOX_CORE_STATICFILES_STORAGE = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_STATICFILES_STORAGE',
        settings.EOX_CORE_STATICFILES_STORAGE
    )
    if settings.EOX_CORE_ENABLE_STATICFILES_STORAGE:
        settings.STATICFILES_STORAGE = settings.EOX_CORE_STATICFILES_STORAGE

    settings.EOX_CORE_LOAD_PERMISSIONS = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_LOAD_PERMISSIONS',
        settings.EOX_CORE_LOAD_PERMISSIONS
    )
    settings.DATA_API_DEF_PAGE_SIZE = getattr(settings, 'ENV_TOKENS', {}).get(
        'DATA_API_DEF_PAGE_SIZE',
        settings.DATA_API_DEF_PAGE_SIZE
    )
    settings.DATA_API_MAX_PAGE_SIZE = getattr(settings, 'ENV_TOKENS', {}).get(
        'DATA_API_MAX_PAGE_SIZE',
        settings.DATA_API_MAX_PAGE_SIZE
    )
    settings.EOX_CORE_COURSES_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_COURSES_BACKEND',
        settings.EOX_CORE_COURSES_BACKEND
    )
    settings.EOX_CORE_COURSEKEY_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_COURSEKEY_BACKEND',
        settings.EOX_CORE_COURSEKEY_BACKEND
    )
    settings.EOX_CORE_COURSE_MANAGEMENT_REQUEST_TIMEOUT = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_COURSE_MANAGEMENT_REQUEST_TIMEOUT',
        settings.EOX_CORE_COURSE_MANAGEMENT_REQUEST_TIMEOUT
    )
    settings.EOX_CORE_THIRD_PARTY_AUTH_BACKEND = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_THIRD_PARTY_AUTH_BACKEND',
        settings.EOX_CORE_THIRD_PARTY_AUTH_BACKEND
    )
    settings.EOX_CORE_USER_ENABLE_MULTI_TENANCY = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_USER_ENABLE_MULTI_TENANCY',
        settings.EOX_CORE_USER_ENABLE_MULTI_TENANCY
    )
    if not settings.EOX_CORE_USER_ENABLE_MULTI_TENANCY:
        user_origin_sources = [
            'fetch_from_unfiltered_table',
        ]
    else:
        user_origin_sources = settings.EOX_CORE_USER_ORIGIN_SITE_SOURCES
    settings.EOX_CORE_USER_ORIGIN_SITE_SOURCES = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_USER_ORIGIN_SITE_SOURCES',
        user_origin_sources
    )

    settings.EOX_CORE_APPEND_LMS_MIDDLEWARE_CLASSES = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_APPEND_LMS_MIDDLEWARE_CLASSES',
        settings.EOX_CORE_APPEND_LMS_MIDDLEWARE_CLASSES
    )
    if settings.SERVICE_VARIANT == "lms":
        if settings.EOX_CORE_APPEND_LMS_MIDDLEWARE_CLASSES:
            settings.MIDDLEWARE += [
                'eox_core.middleware.PathRedirectionMiddleware',
                'eox_core.middleware.RedirectionsMiddleware',
                'eox_core.middleware.TPAExceptionMiddleware'
            ]

    # Sentry Integration
    sentry_integration_dsn = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_SENTRY_INTEGRATION_DSN',
        settings.EOX_CORE_SENTRY_INTEGRATION_DSN
    )
    settings.EOX_CORE_SENTRY_IGNORED_ERRORS = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_SENTRY_IGNORED_ERRORS',
        settings.EOX_CORE_SENTRY_IGNORED_ERRORS
    )
    sentry_environment = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_SENTRY_ENVIRONMENT',
        settings.EOX_CORE_SENTRY_ENVIRONMENT
    )
    sentry_extra_options = getattr(settings, 'ENV_TOKENS', {}).get(
        'EOX_CORE_SENTRY_EXTRA_OPTIONS',
        settings.EOX_CORE_SENTRY_EXTRA_OPTIONS
    )

    if sentry_sdk is not None and sentry_integration_dsn is not None:
        from eox_core.integrations.sentry import ExceptionFilterSentry  # pylint: disable=import-outside-toplevel
        try:
            sentry_sdk.init(
                before_send=ExceptionFilterSentry(),
                dsn=sentry_integration_dsn,
                environment=sentry_environment,
                integrations=[
                    DjangoIntegration(),
                ],
                send_default_pii=True,
                **sentry_extra_options
            )
        except TypeError:
            sentry_sdk.init(
                before_send=ExceptionFilterSentry(),
                dsn=sentry_integration_dsn,
                environment=sentry_environment,
                integrations=[
                    DjangoIntegration(),
                ],
                send_default_pii=True
            )



"""
API v1 views.
"""

from __future__ import absolute_import, unicode_literals

import logging

from django.conf import settings
from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.sites.shortcuts import get_current_site
from django.db import transaction
from edx_rest_framework_extensions.auth.jwt.authentication import JwtAuthentication
from oauth2_provider.models import Application
from rest_framework import status
from rest_framework.authentication import SessionAuthentication
from rest_framework.exceptions import NotFound
from rest_framework.renderers import BrowsableAPIRenderer, JSONRenderer
from rest_framework.response import Response
from rest_framework.views import APIView

from eox_core.api.support.v1.permissions import EoxCoreSupportAPIPermission
from eox_core.api.support.v1.serializers import (
    OauthApplicationSerializer,
    WrittableEdxappRemoveUserSerializer,
    WrittableEdxappUsernameSerializer,
)
from eox_core.api.v1.serializers import EdxappUserReadOnlySerializer
from eox_core.api.v1.views import UserQueryMixin
from eox_core.edxapp_wrapper.bearer_authentication import BearerAuthentication
from eox_core.edxapp_wrapper.comments_service_users import replace_username_cs_user
from eox_core.edxapp_wrapper.users import (
    create_edxapp_user,
    delete_edxapp_user,
    get_edxapp_user,
    get_user_signup_source,
)
from eox_core.utils import get_or_create_site_from_oauth_app_uris

User = get_user_model()
UserSignupSource = get_user_signup_source()  # pylint: disable=invalid-name

try:
    from eox_audit_model.decorators import audit_drf_api
except ImportError:
    def audit_drf_api(*args, **kwargs):
        """Identity decorator"""
        return lambda x: x

LOG = logging.getLogger(__name__)


class EdxappUser(UserQueryMixin, APIView):
    """
    Handles API requests to remove users
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication, JwtAuthentication)
    permission_classes = (EoxCoreSupportAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    @audit_drf_api(action='Remove edxapp User.', method_name='eox_core_api_method')
    def delete(self, request, *args, **kwargs):  # pylint: disable=too-many-locals
        """
        Allows to safely remove an edxapp User.

        For example:

        **Requests**:
            DELETE <domain>/eox-core/support-api/v1/user/

        **Request body**:
            {
                "case_id": Optional. ID of the support case for naming the retired user Email
            }

        **Response values**:
            - 200: Success, user has multiple signup sources and the one that matches the current site
                    has been deleted.
            - 202: Success, user has a unique signup source and has been safely removed from the platform.
            - 400: Bad request, invalid case_id or user has no signup source on the request site.
        """
        query = self.get_user_query(request)

        serializer = WrittableEdxappRemoveUserSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        data = serializer.validated_data
        data["site"] = get_current_site(request)
        data["user"] = get_edxapp_user(**query)

        message, status = delete_edxapp_user(**data)  # pylint: disable=redefined-outer-name

        return Response(message, status=status)


class EdxappReplaceUsername(UserQueryMixin, APIView):
    """
    Handles the replacement of the username.
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication, JwtAuthentication)
    permission_classes = (EoxCoreSupportAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    @audit_drf_api(action="Update an Edxapp user's Username.", method_name='eox_core_api_method')
    def patch(self, request, *args, **kwargs):
        """
        Allows to safely update an Edxapp user's Username along with the
        forum associated User.

        For now users that have different signup sources cannot be updated.

        For example:

        **Requests**:
            PATCH <domain>/eox-core/support-api/v1/replace-username/

        **Request body**
            {
                "new_username": "new username"
            }

        **Response values**
            User serialized.
        """
        query = self.get_user_query(request)
        user = get_edxapp_user(**query)
        data = request.data

        with transaction.atomic():
            serializer = WrittableEdxappUsernameSerializer(user, data=data)
            serializer.is_valid(raise_exception=True)
            serializer.save()

            data = serializer.validated_data
            data["user"] = user

            # Update user in cs_comments_service forums
            replace_username_cs_user(**data)

        admin_fields = getattr(settings, "ACCOUNT_VISIBILITY_CONFIGURATION", {}).get(
            "admin_fields", {}
        )
        serialized_user = EdxappUserReadOnlySerializer(
            user, custom_fields=admin_fields, context={"request": request}
        )
        return Response(serialized_user.data)


class OauthApplicationAPIView(UserQueryMixin, APIView):
    """
    Handles requests related to the
    django_oauth_toolkit Application object.
    """

    authentication_classes = (BearerAuthentication, SessionAuthentication, JwtAuthentication)
    permission_classes = (EoxCoreSupportAPIPermission,)
    renderer_classes = (JSONRenderer, BrowsableAPIRenderer)

    @audit_drf_api(action='Generate Oauth Application.', method_name='eox_core_api_method')
    def post(self, request, *args, **kwargs):  # pylint: disable=too-many-locals
        """
        Creates a new Oauth Application from django_oauth_toolkit.

        This endpoint assumes that the URLs sent in the redirect_uris
        parameter are correct.

        In order to generate a valid application, this endpoint
        does multiple things:
        1 - Get or Create an edxapp user which will be the owner
        of the Application.
        2 - Grant permissions to the user that were sent in the
        "permissions" list.
        3 - Create User SignUp Source with the created user and the site
            sent in the redirect_uris field.
        4 - Create Application.

        Note: make sure to send the codenames of the permissions
        you want to grant to the user in the permissions list.
        Also, send the urls in the redirect_uris separated by
        simple whitespaces to avoid any errors during the creation.

        For example:

        **Requests**:
            POST <domain>/eox-core/support-api/v1/oauth-application/

        **Request body**:
        {
            "user": {
                "fullname": "John Doe",
                "email": "johndoe@example.com",
                "username": "johndoe",
                "permissions": ["can_call_eox_core", "can_call_eox_tenant"]
            },
            "redirect_uris": "http://testing-site.io/ http://testing-site.io",
            "client_type":"confidential",
            "authorization_grant_type":"client-credentials",
            "name": "test-application",
            "skip_authorization": true
        }

        **Response values**:
            - 200: Success, the Oauth Application has been created.
            - 400: Bad request, invalid request body format.
            - 500: The server has failed to get or create the.
            owner user for the application.
        """
        message = "Could not get or create edxapp User"

        serializer = OauthApplicationSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        data = serializer.validated_data
        user_creation_data = data.pop('user', {})
        user_permissions = user_creation_data.pop('permissions', [])
        site = get_or_create_site_from_oauth_app_uris(data.get('redirect_uris', ''))

        user_creation_data.update({
            'skip_extra_registration_fields': True,
            'activate_user': True,
            'skip_password': True,
            'site': site,
        })

        # Get or create user
        try:
            user = get_edxapp_user(**user_creation_data)
        except (NotFound, User.DoesNotExist):
            user, message = create_edxapp_user(**user_creation_data)

        if not user:
            LOG.error(message)

            return Response(message, status=status.HTTP_500_INTERNAL_SERVER_ERROR)

        # Create SignUp Source in the Oauth Application site
        UserSignupSource.objects.get_or_create(user=user, site=site.name)

        # Grant permissions to user
        permissions = Permission.objects.filter(codename__in=user_permissions)
        user.user_permissions.add(*permissions)

        # Create Oauth Application
        data['user'] = user
        application, _ = Application.objects.get_or_create(**data)

        return Response(OauthApplicationSerializer(application).data, status=status.HTTP_200_OK)



"""
Test module for the OauthApplicationAPIView class.
"""
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site
from django.urls import reverse
from mock import Mock, patch
from oauth2_provider.models import Application
from rest_framework import status
from rest_framework.test import APIClient, APITestCase

from eox_core.api.support.v1.views import User


class OauthApplicationAPIViewTest(APITestCase):  # pylint: disable=too-many-instance-attributes
    """Oauth Application API TestCase."""

    def setUp(self):
        """
        setup.
        """
        super().setUp()
        self.api_user = User(
            username='staff',
            email='staffuser@example.com',
            is_staff=True,
        )
        self.url = reverse('eox-support-api:eox-support-api:edxapp-oauth-application')
        self.client = APIClient()
        self.client.force_authenticate(user=self.api_user)
        self.site = Site.objects.create(
            domain='testing-site.edunext.io',
            name='testing-site',
        )
        self.user = User.objects.create(
            username='johndoe',
            email='johndoe@example.com',
        )
        self.content_type = ContentType.objects.get_for_model(User)
        self.permission_test_1 = Permission.objects.get_or_create(  # pylint: disable=unused-variable
            codename='test_1',
            name='Can access test-1 API',
            content_type=self.content_type,
        )
        self.permission_test_2 = Permission.objects.get_or_create(  # pylint: disable=unused-variable
            codename='test_2',
            name='Can access test-2 API',
            content_type=self.content_type,
        )

    @patch('eox_core.api.support.v1.views.get_or_create_site_from_oauth_app_uris')
    @patch('eox_core.api.support.v1.views.UserSignupSource')
    @patch('eox_core.api.support.v1.views.create_edxapp_user')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    def test_create_oauth_application_with_new_user(
        self,
        mock_get_edxapp_user,
        mock_create_edxapp_user,
        mock_get_user_signup_source,
        mock_get_or_create_site,
    ):
        """Tests the case where an Oauth Application is created
        with a new user.

        Expected behavior:
            - The Oauth App and the User instances are created.
            - Status code 200.
        """
        mock_get_or_create_site.return_value = self.site
        mock_get_edxapp_user.side_effect = User.DoesNotExist
        mock_create_edxapp_user.return_value = self.user, ""
        mock_get_user_signup_source.return_value = Mock()
        data = {
            "user": {
                "fullname": "John Doe",
                "email": "johndoe@example.com",
                "username": "johndoe",
                "permissions": ["test_1", "test_2"]
            },
            "redirect_uris": "http://testing-site.io/ http://testing-site.io",
            "client_type": "confidential",
            "authorization_grant_type": "client-credentials",
            "name": "test-application",
            "skip_authorization": True,
        }

        response = self.client.post(self.url, data=data, format="json")

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertTrue(data["name"], response.data["name"])
        self.assertTrue(self.user.user_permissions.filter(codename="test_1").exists())
        self.assertTrue(self.user.user_permissions.filter(codename="test_2").exists())
        mock_get_edxapp_user.assert_called_once()
        mock_create_edxapp_user.assert_called_once()
        mock_get_or_create_site.assert_called_once()

    @patch('eox_core.api.support.v1.views.get_or_create_site_from_oauth_app_uris')
    @patch('eox_core.api.support.v1.views.UserSignupSource')
    @patch('eox_core.api.support.v1.views.create_edxapp_user')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    def test_create_oauth_application_with_existing_user_and_permissions(
        self,
        mock_get_edxapp_user,
        mock_create_edxapp_user,
        mock_get_user_signup_source,
        mock_get_or_create_site,
    ):
        """Tests the case where an Oauth Application is created
        with an existing user.

        Expected behavior:
            - The Oauth App is created with the existing user.
            - Status code 200.
        """
        mock_get_or_create_site.return_value = self.site
        mock_get_edxapp_user.return_value = self.user
        mock_get_user_signup_source.return_value = Mock()
        data = {
            "user": {
                "fullname": "John Doe",
                "email": "johndoe@example.com",
                "username": "johndoe",
                "permissions": ["test_1", "test_2"]
            },
            "redirect_uris": "http://testing-site2.io/ http://testing-site2.io",
            "client_type": "confidential",
            "authorization_grant_type": "client-credentials",
            "name": "test-application 2",
            "skip_authorization": True,
        }

        response = self.client.post(self.url, data=data, format="json")

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertTrue(data["name"], response.data["name"])
        self.assertTrue(self.user.user_permissions.filter(codename="test_1").exists())
        self.assertTrue(self.user.user_permissions.filter(codename="test_2").exists())
        self.assertEqual(2, self.user.user_permissions.all().count())
        mock_get_edxapp_user.assert_called_once()
        mock_create_edxapp_user.assert_not_called()
        mock_get_or_create_site.assert_called_once()

    @patch('eox_core.api.support.v1.views.get_or_create_site_from_oauth_app_uris')
    @patch('eox_core.api.support.v1.views.create_edxapp_user')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    def test_create_oauth_application_with_wrong_data(
        self,
        mock_get_edxapp_user,
        mock_create_edxapp_user,
        mock_get_or_create_site,
    ):
        """Tests the case where the data sended in the
        request is incomplete or incorrect.

        Expected behavior:
            - Status code 400 BAD REQUEST.
        """
        message = b'{"authorization_grant_type":["This field is required."]}'
        mock_get_or_create_site.return_value = self.site
        mock_get_edxapp_user.return_value = self.user
        data = {
            "user": {
                "fullname": "John Doe",
                "email": "johndoe@example.com",
                "username": "johndoe",
                "permissions": ["test_1", "test_2"]
            },
            "client_type": "confidential",
            "skip_authorization": True,
        }

        response = self.client.post(self.url, data=data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(message, response.content)
        mock_get_edxapp_user.assert_not_called()
        mock_create_edxapp_user.assert_not_called()
        mock_get_or_create_site.assert_not_called()

    @patch('eox_core.api.support.v1.views.get_or_create_site_from_oauth_app_uris')
    @patch('eox_core.api.support.v1.views.UserSignupSource')
    @patch('eox_core.api.support.v1.views.create_edxapp_user')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    def test_create_oauth_application_with_wrong_permissions_for_user(
        self,
        mock_get_edxapp_user,
        mock_create_edxapp_user,
        mock_get_user_signup_source,
        mock_get_or_create_site,
    ):
        """Tests the case where the permissions sent for the user
        don't exist.

        Expected behavior:
            - The Oauth App and the User instances are created,
            but the user has no permissions granted.
            - Status code 200.
        """
        mock_get_or_create_site.return_value = self.site
        mock_get_edxapp_user.return_value = self.user
        mock_get_user_signup_source.return_value = Mock()
        data = {
            "user": {
                "fullname": "John Doe",
                "email": "johndoe@example.com",
                "username": "johndoe",
                "permissions": ["test_7", "test_8"]
            },
            "redirect_uris": "http://testing-site2.io/ http://testing-site2.io",
            "client_type": "confidential",
            "authorization_grant_type": "client-credentials",
            "name": "test-application 3",
            "skip_authorization": True,
        }

        response = self.client.post(self.url, data=data, format="json")

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertTrue(data["name"], response.data["name"])
        self.assertFalse(self.user.user_permissions.filter(codename="test_7").exists())
        self.assertFalse(self.user.user_permissions.filter(codename="test_8").exists())
        mock_get_edxapp_user.assert_called_once()
        mock_create_edxapp_user.assert_not_called()
        mock_get_or_create_site.assert_called_once()

    @patch('eox_core.api.support.v1.views.get_or_create_site_from_oauth_app_uris')
    @patch('eox_core.api.support.v1.views.UserSignupSource')
    @patch('eox_core.api.support.v1.views.create_edxapp_user')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    def test_get_existing_oauth_application(
        self,
        mock_get_edxapp_user,
        mock_create_edxapp_user,
        mock_get_user_signup_source,
        mock_get_or_create_site,
    ):
        """Tests the case an Oauth Application with the same
        information already exists.

        Expected behavior:
            - Returns the existent Oauth Application.
            - Status code 200.
        """
        mock_get_or_create_site.return_value = self.site
        mock_get_edxapp_user.return_value = self.user
        mock_get_user_signup_source.return_value = Mock()
        data = {
            "user": {
                "fullname": "John Doe",
                "email": "johndoe@example.com",
                "username": "johndoe",
                "permissions": ["test_1", "test_2"]
            },
            "redirect_uris": "http://testing-site.io/ http://testing-site.io",
            "client_type": "confidential",
            "authorization_grant_type": "client-credentials",
            "name": "test-application",
            "skip_authorization": True,
        }

        response = self.client.post(self.url, data=data, format="json")

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertTrue(data["name"], response.data["name"])
        self.assertEqual(1, Application.objects.filter(name=data["name"]).count())
        mock_get_edxapp_user.assert_called_once()
        mock_create_edxapp_user.assert_not_called()
        mock_get_or_create_site.assert_called_once()

    @patch('eox_core.api.support.v1.views.get_or_create_site_from_oauth_app_uris')
    @patch('eox_core.api.support.v1.views.create_edxapp_user')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    def test_create_oauth_application_when_get_or_create_edxapp_user_fails(
        self,
        mock_get_edxapp_user,
        mock_create_edxapp_user,
        mock_get_or_create_site,
    ):
        """Tests the case where the methods the get or create
        the edxapp user fails.

        Expected behavior:
            - The Oauth App is not created.
            - Status code 500.
        """
        mock_get_or_create_site.return_value = self.site
        mock_get_edxapp_user.side_effect = User.DoesNotExist
        mock_create_edxapp_user.return_value = None, ""
        data = {
            "user": {
                "fullname": "John Doe",
                "email": "johndoe@example.com",
                "username": "johndoe",
                "permissions": ["test_1", "test_2"]
            },
            "redirect_uris": "http://testing-site.io/ http://testing-site.io",
            "client_type": "confidential",
            "authorization_grant_type": "client-credentials",
            "name": "test-application 4",
            "skip_authorization": True,
        }

        response = self.client.post(self.url, data=data, format="json")

        self.assertEqual(status.HTTP_500_INTERNAL_SERVER_ERROR, response.status_code)
        self.assertFalse(Application.objects.filter(name=data["name"]).exists())
        mock_get_edxapp_user.assert_called_once()
        mock_create_edxapp_user.assert_called_once()
        mock_get_or_create_site.assert_called_once()

    @patch('eox_core.api.support.v1.views.get_or_create_site_from_oauth_app_uris')
    @patch('eox_core.api.support.v1.views.create_edxapp_user')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    def test_create_oauth_application_with_wrong_user_data(
        self,
        mock_get_edxapp_user,
        mock_create_edxapp_user,
        mock_get_or_create_site,
    ):
        """Tests the case where the user information
        is incomplete or has the wrong format in the
        request data.

        Expected behavior:
            - Status code 400 Bad Request.
        """
        message = b'{"user":{"username":["This field is required."]}}'
        mock_get_or_create_site.return_value = self.site
        mock_get_edxapp_user.return_value = self.user
        data = {
            "user": {
                "fullname": "John Doe",
                "email": "johndoe@example.com",
                "permissions": ["test_1", "test_2"]
            },
            "redirect_uris": "http://testing-site.io/ http://testing-site.io",
            "client_type": "confidential",
            "authorization_grant_type": "client-credentials",
            "name": "test-application 5",
            "skip_authorization": True,
        }

        response = self.client.post(self.url, data=data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(message, response.content)
        self.assertFalse(Application.objects.filter(name=data["name"]).exists())
        mock_get_edxapp_user.assert_not_called()
        mock_create_edxapp_user.assert_not_called()
        mock_get_or_create_site.assert_not_called()

    @patch('eox_core.api.support.v1.views.get_or_create_site_from_oauth_app_uris')
    @patch('eox_core.api.support.v1.views.UserSignupSource')
    @patch('eox_core.api.support.v1.views.create_edxapp_user')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    def test_create_oauth_application_without_sending_user_permissions(
        self,
        mock_get_edxapp_user,
        mock_create_edxapp_user,
        mock_get_user_signup_source,
        mock_get_or_create_site,
    ):
        """Tests the case where an Oauth Application is created
        without sending any permissions to assign to the user.

        Expected behavior:
            - The Oauth App is created.
            - Status code 200.
        """
        self.user.user_permissions.clear()
        mock_get_or_create_site.return_value = self.site
        mock_get_edxapp_user.return_value = self.user
        mock_get_user_signup_source.return_value = Mock()
        data = {
            "user": {
                "fullname": "John Doe",
                "email": "johndoe@example.com",
                "username": "johndoe",
                "permissions": [],
            },
            "redirect_uris": "http://testing-site6.io/ http://testing-site6.io",
            "client_type": "confidential",
            "authorization_grant_type": "client-credentials",
            "name": "test-application 6",
            "skip_authorization": True,
        }

        response = self.client.post(self.url, data=data, format="json")

        self.assertEqual(status.HTTP_200_OK, response.status_code)
        self.assertTrue(data["name"], response.data["name"])
        self.assertEqual(0, self.user.user_permissions.all().count())
        mock_get_edxapp_user.assert_called_once()
        mock_create_edxapp_user.assert_not_called()
        mock_get_or_create_site.assert_called_once()



# -*- coding: utf-8 -*-
from django.db import migrations


def update_contentypes(apps, schema_editor):
    """
    Updates content types.
    We want to have the same content type id, when the model is moved.
    """
    ContentType = apps.get_model('contenttypes', 'ContentType')
    db_alias = schema_editor.connection.alias
    # Move the Redirection to eox_core
    qs = ContentType.objects.using(db_alias).filter(app_label='eox_tenant', model='Redirection')
    qs.update(app_label='eox_core')


def update_contentypes_reverse(apps, schema_editor):
    """
    Reverts changes in content types.
    """
    ContentType = apps.get_model('contenttypes', 'ContentType')
    db_alias = schema_editor.connection.alias
    qs = ContentType.objects.using(db_alias).filter(app_label='eox_core', model='Redirection')
    qs.update(app_label='eox_tenant')


class Migration(migrations.Migration):
    dependencies = [
        ('eox_core', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(update_contentypes, update_contentypes_reverse)
    ]



"""
Test backend to get GenerateCertificates Model.
"""

from django.contrib.auth.models import Permission


def get_generated_certificate():
    """
    Get test GeneratedCertificate model.

    We return any django model that already exists so that
    django-filters is happy and no migrations are created.
    """
    return Permission



"""
Test module for users viewset.
"""
from django.contrib.auth.models import User
from django.test import TestCase
from django.urls import reverse
from mock import patch
from rest_framework import status
from rest_framework.test import APIClient


class EdxappReplaceUsernameAPITest(TestCase):
    """Test class for update username APIView."""

    patch_permissions = patch('eox_core.api.support.v1.permissions.EoxCoreSupportAPIPermission.has_permission', return_value=True)

    def setUp(self):
        """Setup method for test class."""
        self.user = User(username="test-username", email="test-username@example.com", password="testusername")
        self.client = APIClient()
        self.url = reverse("eox-support-api:eox-support-api:edxapp-replace-username")
        self.client.force_authenticate(user=self.user)

    @patch_permissions
    @patch('eox_core.api.support.v1.views.replace_username_cs_user')
    @patch('eox_core.api.support.v1.serializers.UserSignupSource')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    @patch('eox_core.api.support.v1.views.EdxappUserReadOnlySerializer')
    def test_replace_username_success(self, user_serializer, get_edxapp_user, signup_source, replace_username_cs_user, _):
        """Test the replacement of the username of an edxapp user."""
        update_data = {
            "username": self.user.username,
            "new_username": "replaced-username",
        }

        user_serializer.return_value.data = {}
        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertEqual("replaced-username", self.user.username)
        self.assertEqual(status.HTTP_200_OK, response.status_code)

    @patch_permissions
    @patch('eox_core.api.support.v1.serializers.UserSignupSource')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    def test_replace_username_bad_sign_up_source(self, get_edxapp_user, signup_source, _):
        """
        Tests that when a user has more than one signup source then the
        username cannot be replaced.
        """
        update_data = {
            "username": self.user.username,
            "new_username": "another-username",
        }

        get_edxapp_user.return_value = self.user
        signup_source.objects.filter.return_value.count.return_value = 2

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"detail":["You can\'t update users with more than one sign up source."]}'
                         .encode())

    @patch_permissions
    @patch('eox_core.api.support.v1.serializers.UserSignupSource')
    @patch('eox_core.api.support.v1.views.get_edxapp_user')
    def test_replace_username_staff_user(self, get_edxapp_user, signup_source, _):
        """Tests that if a user is staff or superuser then the username cannot be replaced."""
        user = User(username="test", email="test@example.com", password="testtest", is_staff=True)
        update_data = {
            "username": user.username,
            "new_username": "new-test-username",
        }

        get_edxapp_user.return_value = user
        signup_source.objects.filter.return_value.count.return_value = 1

        response = self.client.patch(self.url, data=update_data, format="json")

        self.assertEqual(status.HTTP_400_BAD_REQUEST, response.status_code)
        self.assertEqual(response.content, '{"detail":["You can\'t update users with roles like staff or superuser."]}'
                         .encode())



#!/usr/bin/python
"""
Test module for the custom storages
"""
import mock
from django.test import TestCase

from eox_core.storage import DevelopmentStorage, ProductionStorage


class StaticStoragesTest(TestCase):
    """
    Testing the extended storages
    """
    @mock.patch('eox_core.test_utils.TestStorage.url')
    def test_calling_parent_storage_url_method(self, test_url_mock):  # pylint: disable=invalid-name
        """
        Test the case when the storages have to call the url parent method
        """
        test_url_mock.return_value = "/static/images/logo.png"
        instance_storage = ProductionStorage()
        name = "logo.png"
        result = instance_storage.url(name)
        test_url_mock.assert_called_once()
        self.assertEqual(result, "/static/images/logo.png")

        test_url_mock.reset_mock()
        instance_storage = DevelopmentStorage()
        result = instance_storage.url(name)
        test_url_mock.assert_called_once()
        self.assertEqual(result, "/static/images/logo.png")

    @mock.patch('eox_core.test_utils.TestStorage.url')
    def test_returning_absolute_url_asset(self, test_url_mock):  # pylint: disable=invalid-name
        """
        Test the case when the storages don't have to call the url parent method
        """
        instance_storage = ProductionStorage()
        name = "https://example.com/assets/logo.png"
        result = instance_storage.url(name)
        test_url_mock.assert_not_called()
        self.assertEqual(result, name)

        test_url_mock.reset_mock()
        instance_storage = DevelopmentStorage()
        result = instance_storage.url(name)
        test_url_mock.assert_not_called()
        self.assertEqual(result, name)



"""
Authentication definitions.
"""
from importlib import import_module

from django.conf import settings


def get_bearer_authentication():
    """ Gets BearerAuthentication class. """
    backend_function = settings.EOX_CORE_BEARER_AUTHENTICATION
    backend = import_module(backend_function)

    return backend.get_bearer_authentication()


BearerAuthentication = get_bearer_authentication()  # pylint: disable=invalid-name



#!/usr/bin/python
"""
Util function definitions.
"""
import datetime
import hashlib
import re

from django.conf import settings
from django.contrib.sites.models import Site
from django.core import cache
from pytz import UTC
from rest_framework import serializers

from eox_core.edxapp_wrapper.users import get_user_profile

UserProfile = get_user_profile()

try:
    cache = cache.caches['general']  # pylint: disable=invalid-name
except Exception:  # pylint: disable=broad-except
    cache = cache.cache  # pylint: disable=invalid-name


def fasthash(string):
    """
    Hashes `string` into a string representation of a 128-bit digest.
    """
    md5 = hashlib.new("md5")
    md5.update(string.encode('utf-8'))
    return md5.hexdigest()


def get_valid_years():
    """
    Return valid list of year range, for the YEAR_OF_BIRTH_CHOICES
    constant.
    """
    current_year = datetime.datetime.now(UTC).year
    return list(range(current_year, current_year - 120, -1))


def get_gender_choices():
    """
    Try to return the valid options for the UserProfile field "gender"
    """
    return getattr(UserProfile, "GENDER_CHOICES", ())


def get_level_of_education_choices():
    """
    Try to return the valid options for the UserProfile field "level_of_education".
    """
    return getattr(UserProfile, "LEVEL_OF_EDUCATION_CHOICES", ())


def set_custom_field_restrictions(custom_field, serializer_field):
    """
    Given a custom_field definition dict, check for any
    restriction and add it to the serializer field dictionary

    This function is only called for custom fields of type
    "text". The allowed restrictions are taken from the platform's
    ALLOWED_RESTRICTIONS dictionary

    ALLOWED_RESTRICTIONS = {
        "text": ["min_length", "max_length"],
        "password": ["min_length", "max_length", "min_upper", "min_lower",
                     "min_punctuation", "min_symbol", "min_numeric", "min_alphabetic"],
        "email": ["min_length", "max_length", "readonly"],
    }
    """
    for key, value in custom_field.get("restrictions", {}).items():
        try:
            ["min_length", "max_length"].index(key)
            serializer_field[key] = int(value)
        except ValueError as err:
            raise serializers.ValidationError({"restriction error": f"{key}: {value}.\
            The restriction may not be valid or the value is not an integer"}) from err

    return serializer_field


def set_select_custom_field(custom_field, serializer_field):
    """
    Given a custom_field definition dict with type=='select',
    add its options and default value (if exists) to the serializer
    field dictionary.
    """
    field_name = custom_field.get("name")
    choices = custom_field.get("options", [])
    serializer_field["choices"] = choices
    default = custom_field.get("default")

    if default:
        try:
            choices.index(default)
            serializer_field["default"] = default
            # A field can not be both `required` and have a `default`
            serializer_field["required"] = False
        except ValueError as err:
            raise serializers.ValidationError({f"{field_name}": "The default value must be one of the options"}) from err

    return serializer_field


def get_registration_extra_fields():
    """
    Return only the registration extra fields
    that are not hidden.
    These fields are the ones that will be taken into
    account to initialize the EdxappExtendedUserSerializer
    """
    registration_extra_fields = getattr(settings, 'REGISTRATION_EXTRA_FIELDS', {})

    return {key: value for key, value in registration_extra_fields.items() if value in ["required", "optional"]}


def create_user_profile(user):
    """
    Creates a Profile to a User.
    """
    if not hasattr(user, "profile"):
        UserProfile.objects.create(user=user)


def get_domain_from_oauth_app_uris(redirect_uris):
    """
    Gets the URL without the protocol and the final backslash from the
    first URL in the Oauth Application redirect_uris.

    Args:
        redirect_uris: String that contains multiple urls separated
        by one or more whitespaces.

    Example:
        redirect_uris: "http://cloud-sandbox.co/ http://cloud-sandbox.co"

        returns "cloud-sandbox.co"
    """
    protocol_regex = re.compile(r'https?://(www\.)?')
    url = redirect_uris.split()[0]

    return protocol_regex.sub('', url).rstrip('/').rsplit(':', 1)[0]


def get_or_create_site_from_oauth_app_uris(redirect_uris):
    """
    Gets or creates a Django site from the first URL in the Oauth Application
    redirect_uris sent as argument.

    Args:
        redirect_uris: String that contains multiple urls separated
        by one or more whitespaces.

    Example:
        redirect_uris: "http://cloud-sandbox.co/ http://cloud-sandbox.co"

        returns Django Site instance with its domain equal to "cloud-sandbox.co".
    """
    domain = get_domain_from_oauth_app_uris(redirect_uris)
    sites_qs = Site.objects.filter(domain=domain)

    if sites_qs.exists():
        return sites_qs.first()

    return Site.objects.create(domain=domain, name=domain)



"""
Settings for eox-core
"""
from __future__ import absolute_import, unicode_literals

from importlib.util import find_spec

SECRET_KEY = 'a-not-to-be-trusted-secret-key'
INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'edx_proctoring',
    'django_filters',
    'oauth2_provider',
    'django_countries',
)
EOX_AUDIT_MODEL_APP = 'eox_audit_model.apps.EoxAuditModelConfig'


def plugin_settings(settings):
    """
    Defines eox-core settings when app is used as a plugin to edx-platform.
    See: https://github.com/edx/edx-platform/blob/master/openedx/core/djangoapps/plugins/README.rst
    """
    settings.EOX_CORE_COMMENTS_SERVICE_USERS_BACKEND = "eox_core.edxapp_wrapper.backends.comments_service_users_j_v1"
    settings.EOX_CORE_USERS_BACKEND = "eox_core.edxapp_wrapper.backends.users_m_v1"
    settings.EOX_CORE_ENROLLMENT_BACKEND = "eox_core.edxapp_wrapper.backends.enrollment_o_v1"
    settings.EOX_CORE_PRE_ENROLLMENT_BACKEND = "eox_core.edxapp_wrapper.backends.pre_enrollment_l_v1"
    settings.EOX_CORE_CERTIFICATES_BACKEND = "eox_core.edxapp_wrapper.backends.certificates_m_v1"
    settings.EOX_CORE_CONFIGURATION_HELPER_BACKEND = "eox_core.edxapp_wrapper.backends.configuration_helpers_h_v1"
    settings.EOX_CORE_COURSEWARE_BACKEND = "eox_core.edxapp_wrapper.backends.courseware_h_v1"
    settings.EOX_CORE_GRADES_BACKEND = "eox_core.edxapp_wrapper.backends.grades_h_v1"
    settings.EOX_CORE_STORAGES_BACKEND = "eox_core.edxapp_wrapper.backends.storages_i_v1"
    settings.EOX_CORE_ENABLE_STATICFILES_STORAGE = False
    settings.EOX_CORE_STATICFILES_STORAGE = "eox_core.storage.ProductionStorage"
    settings.EOX_CORE_LOAD_PERMISSIONS = True
    settings.DATA_API_DEF_PAGE_SIZE = 1000
    settings.DATA_API_MAX_PAGE_SIZE = 5000
    settings.EOX_CORE_COURSES_BACKEND = "eox_core.edxapp_wrapper.backends.courses_h_v1"
    settings.EOX_CORE_COURSEKEY_BACKEND = "eox_core.edxapp_wrapper.backends.coursekey_m_v1"
    settings.EOX_CORE_COURSE_MANAGEMENT_REQUEST_TIMEOUT = 1000
    settings.EOX_CORE_USER_ENABLE_MULTI_TENANCY = True
    settings.EOX_CORE_USER_ORIGIN_SITE_SOURCES = ['fetch_from_unfiltered_table', ]
    settings.EOX_CORE_APPEND_LMS_MIDDLEWARE_CLASSES = False
    settings.EOX_CORE_ENABLE_UPDATE_USERS = True
    settings.EOX_CORE_USER_UPDATE_SAFE_FIELDS = ["is_active", "password", "fullname", "mailing_address", "year_of_birth", "gender", "level_of_education", "city", "country", "goals", "bio", "phone_number"]
    settings.EOX_CORE_BEARER_AUTHENTICATION = 'eox_core.edxapp_wrapper.backends.bearer_authentication_j_v1'
    settings.EOX_CORE_ASYNC_TASKS = []
    settings.EOX_CORE_THIRD_PARTY_AUTH_BACKEND = 'eox_core.edxapp_wrapper.backends.third_party_auth_l_v1'

    if settings.EOX_CORE_USER_ENABLE_MULTI_TENANCY:
        settings.EOX_CORE_USER_ORIGIN_SITE_SOURCES = [
            'fetch_from_created_on_site_prop',
            'fetch_from_user_signup_source',
        ]

    # Sentry Integration
    settings.EOX_CORE_SENTRY_INTEGRATION_DSN = None

    # The setting EOX_CORE_SENTRY_IGNORED_ERRORS is a list of rules that defines which exceptions to ignore.
    # An example below:
    # EOX_CORE_SENTRY_IGNORED_ERRORS = [
    #     {
    #         "exc_class": "openedx.core.djangoapps.user_authn.exceptions.AuthFailedError",
    #         "exc_text": ["AuthFailedError.*Email or password is incorrect"]
    #     },
    # ]
    # Every rule support only 2 keys for now:
    # - exc_class: the path to the exception class we want to ignore. It can only be one
    # - exc_text: a list of regex expressions to search on the last traceback frame text of the exception

    # In this example we have only one rule. We are ignoring AuthFailedError exceptions whose traceback text
    # has a match with the regex provided in the exc_text unique element. If exc_text contains more than one
    # regex, the exception is ignored if any of the regex matches the traceback text.
    settings.EOX_CORE_SENTRY_IGNORED_ERRORS = []
    settings.EOX_CORE_SENTRY_ENVIRONMENT = None
    settings.EOX_CORE_SENTRY_EXTRA_OPTIONS = {}

    if find_spec('eox_audit_model') and EOX_AUDIT_MODEL_APP not in settings.INSTALLED_APPS:
        settings.INSTALLED_APPS.append(EOX_AUDIT_MODEL_APP)



#!/usr/bin/python
"""
Tests for the pipeline module used in third party auth.
"""
from django.contrib.auth.models import User
from django.test import TestCase
from mock import MagicMock, PropertyMock, patch

from eox_core.edxapp_wrapper.users import get_user_signup_source
from eox_core.pipeline import (
    assert_user_information,
    check_disconnect_pipeline_enabled,
    create_signup_source_for_new_association,
    ensure_new_user_has_usable_password,
    ensure_user_has_profile,
    ensure_user_has_signup_source,
)

UserSignupSource = get_user_signup_source()  # pylint: disable=invalid-name


class EnsureUserPasswordUsableTest(TestCase):
    """
    Test the ensure_new_user_has_usable_password pipeline step.
    """
    def setUp(self):
        self.backend_mock = MagicMock()
        self.user_mock = MagicMock(spec=User)

    @patch('eox_core.pipeline.get_user_attribute')
    def test_new_user_gets_usable_password(self, get_user_attribute_mock):
        """
        A new user with an unusable password will get a new password.
        """
        user_attribute_mock = MagicMock()
        get_user_attribute_mock.return_value = user_attribute_mock
        self.user_mock.has_usable_password.return_value = False
        ensure_new_user_has_usable_password(self.backend_mock, user=self.user_mock, is_new=True)
        self.user_mock.save.assert_called()
        user_attribute_mock.set_user_attribute.assert_called()

    def test_new_user_already_with_usable_password(self):
        """
        A new user that already has an usable password won't be modified.
        """
        self.user_mock.has_usable_password.return_value = True
        ensure_new_user_has_usable_password(self.backend_mock, user=self.user_mock, is_new=True)
        self.user_mock.save.assert_not_called()

    def test_non_new_user(self):
        """
        A non new user won't be modified by this step.
        """
        ensure_new_user_has_usable_password(self.backend_mock, user=self.user_mock, is_new=False)
        self.user_mock.save.assert_not_called()


class EnsureUserProfileTest(TestCase):
    """
    Test the custom association backend.
    """
    def setUp(self):
        self.backend_mock = MagicMock()
        self.user_mock = MagicMock()

    @patch('eox_core.edxapp_wrapper.users.import_module')
    def test_user_with_profile_works(self, import_mock):
        """
        A user that already has a profile will do nothing
        """
        backend = MagicMock()
        import_mock.side_effect = backend

        ensure_user_has_profile(self.backend_mock, {}, user=self.user_mock)
        backend().get_user_profile().assert_not_called()

    @patch('eox_core.edxapp_wrapper.users.import_module')
    def test_user_without_profile_works(self, import_mock):
        """
        A user that has no profile will create one
        """
        backend = MagicMock()
        import_mock.side_effect = backend
        backend().get_user_profile().DoesNotExist = ValueError
        type(self.user_mock).profile = PropertyMock(side_effect=ValueError)

        ensure_user_has_profile(self.backend_mock, {}, user=self.user_mock)
        backend().get_user_profile().objects.create.assert_called()


class DisconnectionPipelineTest(TestCase):
    """Test disconnection from TPA provider."""

    def setUp(self):
        self.backend_mock = MagicMock()

    def test_disable_disconnect_pipeline(self):
        """
        Test disabling disconnection pipeline through TPA provider settings.
        """
        self.backend_mock.setting.return_value.get.return_value = True

        with self.assertRaises(Exception):
            check_disconnect_pipeline_enabled(self.backend_mock)

    def test_disconnect_pipeline_enable_explicit(self):
        """
        Test explicitly enable disconnection pipeline through TPA provider settings.
        """
        self.backend_mock.setting.return_value.get.return_value = False

        self.assertIsNone(check_disconnect_pipeline_enabled(self.backend_mock))

    def test_disconnect_pipeline_enable_implicit(self):
        """
        Test enable disconnection pipeline by not defining disable setting through TPA
        provider settings.
        """
        self.backend_mock.setting.return_value.get.return_value = None

        self.assertIsNone(check_disconnect_pipeline_enabled(self.backend_mock))


class ConnectionPipelineTest(TestCase):
    """Test custom pipeline steps for connection to a TPA provider."""

    def setUp(self):
        self.backend_mock = MagicMock()
        self.user_mock = MagicMock(spec=User)
        self.user_mock.email = "test.example.com"
        self.user_mock.username = "test"

    def test_connect_matching_users(self):
        """
        Test connection pipeline when the information from the user from the LMS matches
        the information returned by the TPA provider.
        """
        details = {
            "email": "test.example.com",
            "username": "test",
        }
        self.backend_mock.setting.return_value.get.return_value = ["email", "username"]

        self.assertIsNone(assert_user_information(details, self.user_mock, self.backend_mock))

    def test_connect_unmatching_users(self):
        """
        Test connection pipeline when user's information from the LMS does not match
        the information returned by the TPA provider.
        """
        details = {
            "email": "test.notexample.com",
            "username": "test",
        }
        self.backend_mock.setting.return_value.get.return_value = ["email", "username"]

        with self.assertRaises(Exception):
            assert_user_information(details, self.user_mock, self.backend_mock)

    def test_connect_any_user(self):
        """
        Test connection pipeline when matching fields are not defined. This means that
        does not matter if the users match or not.
        """
        details = {
            "email": "test.notExample.com",
            "username": "testExample",
        }
        self.backend_mock.setting.return_value.get.return_value = []

        self.assertIsNone(assert_user_information(details, self.user_mock, self.backend_mock))


class SignupSourceRegistration(TestCase):
    """Test custom pipeline steps for signup source registration."""

    def setUp(self):
        self.user_mock = MagicMock()
        self.site_mock = MagicMock()

    @patch("eox_core.pipeline.UserSignupSource")
    @patch("eox_core.pipeline.get_current_request")
    def test_signup_source_after_assoc(self, get_request_mock, signup_source_mock):
        """
        This method tests creating a signup source after associating
        a social auth link to the user.

        Expected behavior:
            The signup source is created using the current site and user.
        """
        signup_source_mock.objects.get_or_create.return_value = (MagicMock(), True,)
        get_request_mock.return_value.site = self.site_mock
        kwargs = {
            "new_association": True,
        }

        create_signup_source_for_new_association(self.user_mock, **kwargs)

        signup_source_mock.objects.get_or_create.assert_called_once_with(
            user=self.user_mock,
            site=self.site_mock,
        )

    @patch("eox_core.pipeline.UserSignupSource")
    @patch("eox_core.pipeline.get_current_request")
    def test_ensure_signup_source(self, get_request_mock, signup_source_mock):
        """
        This method tests creating a signup source when the user has a social
        link but does not have the signup source for the current site.

        Expected behavior:
            The signup source is created using the current site and user.
        """
        signup_source_mock.objects.get_or_create.return_value = (MagicMock(), True,)
        get_request_mock.return_value.site = self.site_mock
        kwargs = {}

        ensure_user_has_signup_source(self.user_mock, **kwargs)

        signup_source_mock.objects.get_or_create.assert_called_once_with(
            user=self.user_mock,
            site=self.site_mock,
        )

    @patch("eox_core.pipeline.UserSignupSource")
    @patch("eox_core.pipeline.get_current_request")
    def test_existent_signup_source_after_assoc(self, get_request_mock, signup_source_mock):
        """
        This method tests executing the signup source steps when the user
        already has a signup source for the current site. This is done after a new
        social auth link is created.

        Expected behavior:
            No signup sources are created.
        """
        signup_source_mock.objects.get_or_create.return_value = (MagicMock(), False,)
        get_request_mock.return_value.site = self.site_mock
        kwargs = {
            "new_association": True,
        }

        create_signup_source_for_new_association(self.user_mock, **kwargs)

        signup_source_mock.objects.get_or_create.assert_called_once_with(
            user=self.user_mock,
            site=self.site_mock,
        )

    @patch("eox_core.pipeline.UserSignupSource")
    @patch("eox_core.pipeline.get_current_request")
    def test_ensure_existent_signup_source(self, get_request_mock, signup_source_mock):
        """
        This method tests executing the signup source steps when the user
        already has a signup source for the current site.

        Expected behavior:
            No signup sources are created.
        """
        signup_source_mock.objects.get_or_create.return_value = (MagicMock(), False,)
        get_request_mock.return_value.site = self.site_mock
        kwargs = {}

        ensure_user_has_signup_source(self.user_mock, **kwargs)

        signup_source_mock.objects.get_or_create.assert_called_once_with(
            user=self.user_mock,
            site=self.site_mock,
        )

    @patch("eox_core.pipeline.UserSignupSource")
    @patch("eox_core.pipeline.get_current_request")
    def test_signup_source_during_register_assoc(self, _, signup_source_mock):
        """
        This method tests executing signup source step create_signup_source_for_new_association
        during the registration process.

        Expected behavior:
            No signup sources are created.
        """
        kwargs = {
            "is_new": True,
        }

        create_signup_source_for_new_association(self.user_mock, **kwargs)

        signup_source_mock.objects.get_or_create.get_or_create.assert_not_called()

    @patch("eox_core.pipeline.UserSignupSource")
    @patch("eox_core.pipeline.get_current_request")
    def test_ensure_signup_source_during_register(self, _, signup_source_mock):
        """
        This method tests executing signup source step ensure_user_has_signup_source
        during the registration process.

        Expected behavior:
            No signup sources are created.
        """
        kwargs = {
            "is_new": True,
        }

        ensure_user_has_signup_source(self.user_mock, **kwargs)

        signup_source_mock.objects.get_or_create.assert_not_called()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Custom Support API permissions module
"""
from rest_framework import exceptions, permissions


class EoxCoreSupportAPIPermission(permissions.BasePermission):
    """
    Defines a custom permissions to access eox-core API
    These permissions make sure that a token is created with the client credentials of the same site is being used on.
    """

    def has_permission(self, request, view):
        """
        For now, to grant access only checks if the requesting user:
            1) is_staff
        """
        if request.user.is_staff:
            return True

        # To prevent leaking important information we return the most basic message.
        raise exceptions.NotAuthenticated(detail="Invalid token")
